# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import os

import pytest
import nipype.interfaces.matlab as mlab

matlab_cmd = mlab.get_matlab_command()
no_matlab = matlab_cmd is None
if not no_matlab:
    mlab.MatlabCommand.set_default_matlab_cmd(matlab_cmd)


def clean_workspace_and_get_default_script_file():
    # Make sure things are clean.
    default_script_file = mlab.MatlabInputSpec().script_file
    if os.path.exists(default_script_file):
        os.remove(
            default_script_file
        )  # raise Exception('Default script file needed for tests; please remove %s!' % default_script_file)
    return default_script_file


@pytest.mark.skipif(no_matlab, reason="matlab is not available")
def test_cmdline():
    default_script_file = clean_workspace_and_get_default_script_file()

    mi = mlab.MatlabCommand(
        script='whos', script_file='testscript', mfile=False)

    assert mi.cmdline == \
        matlab_cmd + (' -nodesktop -nosplash -singleCompThread -r "fprintf(1,'
                      '\'Executing code at %s:\\n\',datestr(now));ver,try,'
                      'whos,catch ME,fprintf(2,\'MATLAB code threw an '
                      'exception:\\n\');fprintf(2,\'%s\\n\',ME.message);if '
                      'length(ME.stack) ~= 0, fprintf(2,\'File:%s\\nName:%s\\n'
                      'Line:%d\\n\',ME.stack.file,ME.stack.name,'
                      'ME.stack.line);, end;end;;exit"')

    assert mi.inputs.script == 'whos'
    assert mi.inputs.script_file == 'testscript'
    assert not os.path.exists(
        mi.inputs.script_file), 'scriptfile should not exist'
    assert not os.path.exists(
        default_script_file), 'default scriptfile should not exist.'


@pytest.mark.skipif(no_matlab, reason="matlab is not available")
def test_mlab_inputspec():
    default_script_file = clean_workspace_and_get_default_script_file()
    spec = mlab.MatlabInputSpec()
    for k in [
            'paths', 'script', 'nosplash', 'mfile', 'logfile', 'script_file',
            'nodesktop'
    ]:
        assert k in spec.copyable_trait_names()
    assert spec.nodesktop
    assert spec.nosplash
    assert spec.mfile
    assert spec.script_file == default_script_file


@pytest.mark.skipif(no_matlab, reason="matlab is not available")
def test_mlab_init():
    default_script_file = clean_workspace_and_get_default_script_file()

    assert mlab.MatlabCommand._cmd == 'matlab'
    assert mlab.MatlabCommand.input_spec == mlab.MatlabInputSpec

    assert mlab.MatlabCommand().cmd == matlab_cmd
    mc = mlab.MatlabCommand(matlab_cmd='foo_m')
    assert mc.cmd == 'foo_m'


@pytest.mark.skipif(no_matlab, reason="matlab is not available")
def test_run_interface(tmpdir):
    default_script_file = clean_workspace_and_get_default_script_file()

    mc = mlab.MatlabCommand(matlab_cmd='foo_m')
    assert not os.path.exists(
        default_script_file), 'scriptfile should not exist 1.'
    with pytest.raises(ValueError):
        mc.run()  # script is mandatory
    assert not os.path.exists(
        default_script_file), 'scriptfile should not exist 2.'
    if os.path.exists(default_script_file):  # cleanup
        os.remove(default_script_file)

    mc.inputs.script = 'a=1;'
    assert not os.path.exists(
        default_script_file), 'scriptfile should not exist 3.'
    with pytest.raises(IOError):
        mc.run()  # foo_m is not an executable
    assert os.path.exists(default_script_file), 'scriptfile should exist 3.'
    if os.path.exists(default_script_file):  # cleanup
        os.remove(default_script_file)

    cwd = tmpdir.chdir()

    # bypasses ubuntu dash issue
    mc = mlab.MatlabCommand(script='foo;', paths=[tmpdir.strpath], mfile=True)
    assert not os.path.exists(
        default_script_file), 'scriptfile should not exist 4.'
    with pytest.raises(RuntimeError):
        mc.run()
    assert os.path.exists(default_script_file), 'scriptfile should exist 4.'
    if os.path.exists(default_script_file):  # cleanup
        os.remove(default_script_file)

    # bypasses ubuntu dash issue
    res = mlab.MatlabCommand(
        script='a=1;', paths=[tmpdir.strpath], mfile=True).run()
    assert res.runtime.returncode == 0
    assert os.path.exists(default_script_file), 'scriptfile should exist 5.'
    cwd.chdir()


@pytest.mark.skipif(no_matlab, reason="matlab is not available")
def test_set_matlabcmd():
    default_script_file = clean_workspace_and_get_default_script_file()

    mi = mlab.MatlabCommand()
    mi.set_default_matlab_cmd('foo')
    assert not os.path.exists(
        default_script_file), 'scriptfile should not exist.'
    assert mi._default_matlab_cmd == 'foo'
    mi.set_default_matlab_cmd(matlab_cmd)



#!/usr/bin/env python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
r"""
autosummary_generate.py OPTIONS FILES

Generate automatic RST source files for items referred to in
autosummary:: directives.

Each generated RST file contains a single auto*:: directive which
extracts the docstring of the referred item.

Example Makefile rule::

    generate:
            ./ext/autosummary_generate.py -o source/generated source/*.rst

"""
from __future__ import print_function, unicode_literals
from builtins import open

import re
import inspect
import os
import optparse
import pydoc
from autosummary import import_by_name

try:
    from phantom_import import import_phantom_module
except ImportError:
    import_phantom_module = lambda x: x


def main():
    p = optparse.OptionParser(__doc__.strip())
    p.add_option("-p", "--phantom", action="store", type="string",
                 dest="phantom", default=None,
                 help="Phantom import modules from a file")
    p.add_option("-o", "--output-dir", action="store", type="string",
                 dest="output_dir", default=None,
                 help=("Write all output files to the given directory "
                       "(instead of writing them as specified in the "
                       "autosummary:: directives)"))
    options, args = p.parse_args()

    if len(args) == 0:
        p.error("wrong number of arguments")

    if options.phantom and os.path.isfile(options.phantom):
        import_phantom_module(options.phantom)

    # read
    names = {}
    for name, loc in list(get_documented(args).items()):
        for (filename, sec_title, keyword, toctree) in loc:
            if toctree is not None:
                path = os.path.join(os.path.dirname(filename), toctree)
                names[name] = os.path.abspath(path)

    # write
    for name, path in sorted(names.items()):
        if options.output_dir is not None:
            path = options.output_dir

        if not os.path.isdir(path):
            os.makedirs(path)

        try:
            obj, name = import_by_name(name)
        except ImportError as e:
            print("Failed to import '%s': %s" % (name, e))
            continue

        fn = os.path.join(path, '%s.rst' % name)

        if os.path.exists(fn):
            # skip
            continue

        f = open(fn, 'w')

        try:
            f.write('%s\n%s\n\n' % (name, '=' * len(name)))

            if inspect.isclass(obj):
                if issubclass(obj, Exception):
                    f.write(format_modulemember(name, 'autoexception'))
                else:
                    f.write(format_modulemember(name, 'autoclass'))
            elif inspect.ismodule(obj):
                f.write(format_modulemember(name, 'automodule'))
            elif inspect.ismethod(obj) or inspect.ismethoddescriptor(obj):
                f.write(format_classmember(name, 'automethod'))
            elif callable(obj):
                f.write(format_modulemember(name, 'autofunction'))
            elif hasattr(obj, '__get__'):
                f.write(format_classmember(name, 'autoattribute'))
            else:
                f.write(format_modulemember(name, 'autofunction'))
        finally:
            f.close()


def format_modulemember(name, directive):
    parts = name.split('.')
    mod, name = '.'.join(parts[:-1]), parts[-1]
    return ".. currentmodule:: %s\n\n.. %s:: %s\n" % (mod, directive, name)


def format_classmember(name, directive):
    parts = name.split('.')
    mod, name = '.'.join(parts[:-2]), '.'.join(parts[-2:])
    return ".. currentmodule:: %s\n\n.. %s:: %s\n" % (mod, directive, name)


def get_documented(filenames):
    """
    Find out what items are documented in source/*.rst
    See `get_documented_in_lines`.

    """
    documented = {}
    for filename in filenames:
        f = open(filename, 'r')
        lines = f.read().splitlines()
        documented.update(get_documented_in_lines(lines, filename=filename))
        f.close()
    return documented


def get_documented_in_docstring(name, module=None, filename=None):
    """
    Find out what items are documented in the given object's docstring.
    See `get_documented_in_lines`.

    """
    try:
        obj, real_name = import_by_name(name)
        lines = pydoc.getdoc(obj).splitlines()
        return get_documented_in_lines(lines, module=name, filename=filename)
    except AttributeError:
        pass
    except ImportError as e:
        print("Failed to import '%s': %s" % (name, e))
    return {}


def get_documented_in_lines(lines, module=None, filename=None):
    """
    Find out what items are documented in the given lines

    Returns
    -------
    documented : dict of list of (filename, title, keyword, toctree)
        Dictionary whose keys are documented names of objects.
        The value is a list of locations where the object was documented.
        Each location is a tuple of filename, the current section title,
        the name of the directive, and the value of the :toctree: argument
        (if present) of the directive.

    """
    title_underline_re = re.compile("^[-=*_^#]{3,}\s*$")
    autodoc_re = re.compile(
        ".. auto(function|method|attribute|class|exception|module)::"
        "\s*([A-Za-z0-9_.]+)\s*$")
    autosummary_re = re.compile(r'^\.\.\s+autosummary::\s*')
    module_re = re.compile(
        r'^\.\.\s+(current)?module::\s*([a-zA-Z0-9_.]+)\s*$')
    autosummary_item_re = re.compile(r'^\s+([_a-zA-Z][a-zA-Z0-9_.]*)\s*.*?')
    toctree_arg_re = re.compile(r'^\s+:toctree:\s*(.*?)\s*$')

    documented = {}

    current_title = []
    last_line = None
    toctree = None
    current_module = module
    in_autosummary = False

    for line in lines:
        try:
            if in_autosummary:
                m = toctree_arg_re.match(line)
                if m:
                    toctree = m.group(1)
                    continue

                if line.strip().startswith(':'):
                    continue  # skip options

                m = autosummary_item_re.match(line)
                if m:
                    name = m.group(1).strip()
                    if current_module and not name.startswith(
                            current_module + '.'):
                        name = "%s.%s" % (current_module, name)
                    documented.setdefault(name, []).append(
                        (filename, current_title, 'autosummary', toctree))
                    continue
                if line.strip() == '':
                    continue
                in_autosummary = False

            m = autosummary_re.match(line)
            if m:
                in_autosummary = True
                continue

            m = autodoc_re.search(line)
            if m:
                name = m.group(2).strip()
                if m.group(1) == "module":
                    current_module = name
                    documented.update(get_documented_in_docstring(
                        name, filename=filename))
                elif current_module and not name.startswith(
                        current_module + '.'):
                    name = "%s.%s" % (current_module, name)
                documented.setdefault(name, []).append(
                    (filename, current_title, "auto" + m.group(1), None))
                continue

            m = title_underline_re.match(line)
            if m and last_line:
                current_title = last_line.strip()
                continue

            m = module_re.match(line)
            if m:
                current_module = m.group(2)
                continue
        finally:
            last_line = line

    return documented


if __name__ == "__main__":
    main()



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import ApplyMask


def test_ApplyMask_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-3,
        ),
        invert_xfm=dict(argstr='-invert', ),
        keep_mask_deletion_edits=dict(argstr='-keep_mask_deletion_edits', ),
        mask_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        mask_thresh=dict(argstr='-T %.4f', ),
        out_file=dict(
            argstr='%s',
            extensions=None,
            hash_files=True,
            keep_extension=True,
            name_source=['in_file'],
            name_template='%s_masked',
            position=-1,
        ),
        subjects_dir=dict(),
        transfer=dict(argstr='-transfer %d', ),
        use_abs=dict(argstr='-abs', ),
        xfm_file=dict(
            argstr='-xform %s',
            extensions=None,
        ),
        xfm_source=dict(
            argstr='-lta_src %s',
            extensions=None,
        ),
        xfm_target=dict(
            argstr='-lta_dst %s',
            extensions=None,
        ),
    )
    inputs = ApplyMask.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ApplyMask_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = ApplyMask.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import VBMSegment


def test_VBMSegment_inputs():
    input_map = dict(
        bias_corrected_affine=dict(
            field='estwrite.output.bias.affine',
            usedefault=True,
        ),
        bias_corrected_native=dict(
            field='estwrite.output.bias.native',
            usedefault=True,
        ),
        bias_corrected_normalized=dict(
            field='estwrite.output.bias.warped',
            usedefault=True,
        ),
        bias_fwhm=dict(
            field='estwrite.opts.biasfwhm',
            usedefault=True,
        ),
        bias_regularization=dict(
            field='estwrite.opts.biasreg',
            usedefault=True,
        ),
        cleanup_partitions=dict(
            field='estwrite.extopts.cleanup',
            usedefault=True,
        ),
        csf_dartel=dict(
            field='estwrite.output.CSF.dartel',
            usedefault=True,
        ),
        csf_modulated_normalized=dict(
            field='estwrite.output.CSF.modulated',
            usedefault=True,
        ),
        csf_native=dict(
            field='estwrite.output.CSF.native',
            usedefault=True,
        ),
        csf_normalized=dict(
            field='estwrite.output.CSF.warped',
            usedefault=True,
        ),
        dartel_template=dict(
            extensions=['.hdr', '.img', '.img.gz', '.nii'],
            field='estwrite.extopts.dartelwarp.normhigh.darteltpm',
        ),
        deformation_field=dict(
            field='estwrite.output.warps',
            usedefault=True,
        ),
        display_results=dict(
            field='estwrite.extopts.print',
            usedefault=True,
        ),
        gaussians_per_class=dict(usedefault=True, ),
        gm_dartel=dict(
            field='estwrite.output.GM.dartel',
            usedefault=True,
        ),
        gm_modulated_normalized=dict(
            field='estwrite.output.GM.modulated',
            usedefault=True,
        ),
        gm_native=dict(
            field='estwrite.output.GM.native',
            usedefault=True,
        ),
        gm_normalized=dict(
            field='estwrite.output.GM.warped',
            usedefault=True,
        ),
        in_files=dict(
            copyfile=False,
            field='estwrite.data',
            mandatory=True,
        ),
        jacobian_determinant=dict(
            field='estwrite.jacobian.warped',
            usedefault=True,
        ),
        matlab_cmd=dict(),
        mfile=dict(usedefault=True, ),
        mrf_weighting=dict(
            field='estwrite.extopts.mrf',
            usedefault=True,
        ),
        paths=dict(),
        pve_label_dartel=dict(
            field='estwrite.output.label.dartel',
            usedefault=True,
        ),
        pve_label_native=dict(
            field='estwrite.output.label.native',
            usedefault=True,
        ),
        pve_label_normalized=dict(
            field='estwrite.output.label.warped',
            usedefault=True,
        ),
        sampling_distance=dict(
            field='estwrite.opts.samp',
            usedefault=True,
        ),
        spatial_normalization=dict(usedefault=True, ),
        tissues=dict(
            extensions=['.hdr', '.img', '.img.gz', '.nii'],
            field='estwrite.tpm',
        ),
        use_mcr=dict(),
        use_sanlm_denoising_filter=dict(
            field='estwrite.extopts.sanlm',
            usedefault=True,
        ),
        use_v8struct=dict(
            min_ver='8',
            usedefault=True,
        ),
        warping_regularization=dict(
            field='estwrite.opts.warpreg',
            usedefault=True,
        ),
        wm_dartel=dict(
            field='estwrite.output.WM.dartel',
            usedefault=True,
        ),
        wm_modulated_normalized=dict(
            field='estwrite.output.WM.modulated',
            usedefault=True,
        ),
        wm_native=dict(
            field='estwrite.output.WM.native',
            usedefault=True,
        ),
        wm_normalized=dict(
            field='estwrite.output.WM.warped',
            usedefault=True,
        ),
    )
    inputs = VBMSegment.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_VBMSegment_outputs():
    output_map = dict(
        bias_corrected_images=dict(),
        dartel_input_images=dict(),
        forward_deformation_field=dict(),
        inverse_deformation_field=dict(),
        jacobian_determinant_images=dict(),
        modulated_class_images=dict(),
        native_class_images=dict(),
        normalized_bias_corrected_images=dict(),
        normalized_class_images=dict(),
        pve_label_native_images=dict(),
        pve_label_normalized_images=dict(),
        pve_label_registered_images=dict(),
        transformation_mat=dict(),
    )
    outputs = VBMSegment.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..registration import AffScalarVol


def test_AffScalarVol_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        deformation=dict(
            argstr='-deformation %g %g %g %g %g %g',
            xor=['transform'],
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        euler=dict(
            argstr='-euler %g %g %g',
            xor=['transform'],
        ),
        in_file=dict(
            argstr='-in %s',
            extensions=None,
            mandatory=True,
        ),
        interpolation=dict(
            argstr='-interp %s',
            usedefault=True,
        ),
        out_file=dict(
            argstr='-out %s',
            extensions=None,
            keep_extension=True,
            name_source='in_file',
            name_template='%s_affxfmd',
        ),
        target=dict(
            argstr='-target %s',
            extensions=None,
            xor=['transform'],
        ),
        transform=dict(
            argstr='-trans %s',
            extensions=None,
            xor=['target', 'translation', 'euler', 'deformation'],
        ),
        translation=dict(
            argstr='-translation %g %g %g',
            xor=['transform'],
        ),
    )
    inputs = AffScalarVol.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_AffScalarVol_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = AffScalarVol.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..dti import DTITracker


def test_DTITracker_inputs():
    input_map = dict(
        angle_threshold=dict(argstr='-at %f', ),
        angle_threshold_weight=dict(argstr='-atw %f', ),
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        input_data_prefix=dict(
            argstr='%s',
            position=0,
            usedefault=True,
        ),
        input_type=dict(argstr='-it %s', ),
        invert_x=dict(argstr='-ix', ),
        invert_y=dict(argstr='-iy', ),
        invert_z=dict(argstr='-iz', ),
        mask1_file=dict(
            argstr='-m %s',
            extensions=None,
            mandatory=True,
            position=2,
        ),
        mask1_threshold=dict(position=3, ),
        mask2_file=dict(
            argstr='-m2 %s',
            extensions=None,
            position=4,
        ),
        mask2_threshold=dict(position=5, ),
        output_file=dict(
            argstr='%s',
            extensions=None,
            position=1,
            usedefault=True,
        ),
        output_mask=dict(
            argstr='-om %s',
            extensions=None,
        ),
        primary_vector=dict(argstr='-%s', ),
        random_seed=dict(argstr='-rseed %d', ),
        step_length=dict(argstr='-l %f', ),
        swap_xy=dict(argstr='-sxy', ),
        swap_yz=dict(argstr='-syz', ),
        swap_zx=dict(argstr='-szx', ),
        tensor_file=dict(extensions=None, ),
        tracking_method=dict(argstr='-%s', ),
    )
    inputs = DTITracker.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_DTITracker_outputs():
    output_map = dict(
        mask_file=dict(extensions=None, ),
        track_file=dict(extensions=None, ),
    )
    outputs = DTITracker.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..model import FitGLM


def test_FitGLM_inputs():
    input_map = dict(
        TR=dict(mandatory=True, ),
        drift_model=dict(usedefault=True, ),
        hrf_model=dict(usedefault=True, ),
        mask=dict(extensions=None, ),
        method=dict(usedefault=True, ),
        model=dict(usedefault=True, ),
        normalize_design_matrix=dict(usedefault=True, ),
        plot_design_matrix=dict(usedefault=True, ),
        save_residuals=dict(usedefault=True, ),
        session_info=dict(mandatory=True, ),
    )
    inputs = FitGLM.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_FitGLM_outputs():
    output_map = dict(
        a=dict(extensions=None, ),
        axis=dict(),
        beta=dict(extensions=None, ),
        constants=dict(),
        dof=dict(),
        nvbeta=dict(),
        reg_names=dict(),
        residuals=dict(extensions=None, ),
        s2=dict(extensions=None, ),
    )
    outputs = FitGLM.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)

import nibabel as nb

from ... import logging
from ..base import TraitedSpec, File, isdefined
from .base import DipyDiffusionInterface, DipyBaseInterfaceInputSpec

IFLOGGER = logging.getLogger('nipype.interface')


class APMQballInputSpec(DipyBaseInterfaceInputSpec):
    mask_file = File(exists=True, desc='An optional brain mask')


class APMQballOutputSpec(TraitedSpec):
    out_file = File(exists=True)


class APMQball(DipyDiffusionInterface):
    """
    Calculates the anisotropic power map

    Example
    -------

    >>> import nipype.interfaces.dipy as dipy
    >>> apm = dipy.APMQball()
    >>> apm.inputs.in_file = 'diffusion.nii'
    >>> apm.inputs.in_bvec = 'bvecs'
    >>> apm.inputs.in_bval = 'bvals'
    >>> apm.run()                                   # doctest: +SKIP
    """
    input_spec = APMQballInputSpec
    output_spec = APMQballOutputSpec

    def _run_interface(self, runtime):
        from dipy.reconst import shm
        from dipy.data import get_sphere
        from dipy.reconst.peaks import peaks_from_model

        gtab = self._get_gradient_table()

        img = nb.load(self.inputs.in_file)
        data = img.get_data()
        affine = img.affine
        mask = None
        if isdefined(self.inputs.mask_file):
            mask = nb.load(self.inputs.mask_file).get_data()

        # Fit it
        model = shm.QballModel(gtab, 8)
        sphere = get_sphere('symmetric724')
        peaks = peaks_from_model(
            model=model,
            data=data,
            relative_peak_threshold=.5,
            min_separation_angle=25,
            sphere=sphere,
            mask=mask)
        apm = shm.anisotropic_power(peaks.shm_coeff)
        out_file = self._gen_filename('apm')
        nb.Nifti1Image(apm.astype("float32"), affine).to_filename(out_file)
        IFLOGGER.info('APM qball image saved as %s', out_file)

        return runtime

    def _list_outputs(self):
        outputs = self._outputs().get()
        outputs['out_file'] = self._gen_filename('apm')

        return outputs



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import SurfaceSnapshots


def test_SurfaceSnapshots_inputs():
    input_map = dict(
        annot_file=dict(
            argstr='-annotation %s',
            extensions=None,
            xor=['annot_name'],
        ),
        annot_name=dict(
            argstr='-annotation %s',
            xor=['annot_file'],
        ),
        args=dict(argstr='%s', ),
        colortable=dict(
            argstr='-colortable %s',
            extensions=None,
        ),
        demean_overlay=dict(argstr='-zm', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        hemi=dict(
            argstr='%s',
            mandatory=True,
            position=2,
        ),
        identity_reg=dict(
            argstr='-overlay-reg-identity',
            xor=['overlay_reg', 'identity_reg', 'mni152_reg'],
        ),
        invert_overlay=dict(argstr='-invphaseflag 1', ),
        label_file=dict(
            argstr='-label %s',
            extensions=None,
            xor=['label_name'],
        ),
        label_name=dict(
            argstr='-label %s',
            xor=['label_file'],
        ),
        label_outline=dict(argstr='-label-outline', ),
        label_under=dict(argstr='-labels-under', ),
        mni152_reg=dict(
            argstr='-mni152reg',
            xor=['overlay_reg', 'identity_reg', 'mni152_reg'],
        ),
        orig_suffix=dict(argstr='-orig %s', ),
        overlay=dict(
            argstr='-overlay %s',
            extensions=None,
            requires=['overlay_range'],
        ),
        overlay_range=dict(argstr='%s', ),
        overlay_range_offset=dict(argstr='-foffset %.3f', ),
        overlay_reg=dict(
            argstr='-overlay-reg %s',
            extensions=None,
            xor=['overlay_reg', 'identity_reg', 'mni152_reg'],
        ),
        patch_file=dict(
            argstr='-patch %s',
            extensions=None,
        ),
        reverse_overlay=dict(argstr='-revphaseflag 1', ),
        screenshot_stem=dict(),
        show_color_scale=dict(argstr='-colscalebarflag 1', ),
        show_color_text=dict(argstr='-colscaletext 1', ),
        show_curv=dict(
            argstr='-curv',
            xor=['show_gray_curv'],
        ),
        show_gray_curv=dict(
            argstr='-gray',
            xor=['show_curv'],
        ),
        six_images=dict(),
        sphere_suffix=dict(argstr='-sphere %s', ),
        stem_template_args=dict(requires=['screenshot_stem'], ),
        subject_id=dict(
            argstr='%s',
            mandatory=True,
            position=1,
        ),
        subjects_dir=dict(),
        surface=dict(
            argstr='%s',
            mandatory=True,
            position=3,
        ),
        tcl_script=dict(
            argstr='%s',
            extensions=None,
            genfile=True,
        ),
        truncate_overlay=dict(argstr='-truncphaseflag 1', ),
    )
    inputs = SurfaceSnapshots.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_SurfaceSnapshots_outputs():
    output_map = dict(snapshots=dict(), )
    outputs = SurfaceSnapshots.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import Undump


def test_Undump_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        coordinates_specification=dict(argstr='-%s', ),
        datatype=dict(argstr='-datum %s', ),
        default_value=dict(argstr='-dval %f', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fill_value=dict(argstr='-fval %f', ),
        head_only=dict(argstr='-head_only', ),
        in_file=dict(
            argstr='-master %s',
            copyfile=False,
            extensions=None,
            mandatory=True,
            position=-1,
        ),
        mask_file=dict(
            argstr='-mask %s',
            extensions=None,
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        orient=dict(argstr='-orient %s', ),
        out_file=dict(
            argstr='-prefix %s',
            extensions=None,
            name_source='in_file',
        ),
        outputtype=dict(),
        srad=dict(argstr='-srad %f', ),
    )
    inputs = Undump.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Undump_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = Undump.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..denoising import UnbiasedNonLocalMeans


def test_UnbiasedNonLocalMeans_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        hp=dict(argstr='--hp %f', ),
        inputVolume=dict(
            argstr='%s',
            extensions=None,
            position=-2,
        ),
        outputVolume=dict(
            argstr='%s',
            hash_files=False,
            position=-1,
        ),
        ps=dict(argstr='--ps %f', ),
        rc=dict(
            argstr='--rc %s',
            sep=',',
        ),
        rs=dict(
            argstr='--rs %s',
            sep=',',
        ),
        sigma=dict(argstr='--sigma %f', ),
    )
    inputs = UnbiasedNonLocalMeans.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_UnbiasedNonLocalMeans_outputs():
    output_map = dict(outputVolume=dict(
        extensions=None,
        position=-1,
    ), )
    outputs = UnbiasedNonLocalMeans.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import Fourier


def test_Fourier_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        highpass=dict(
            argstr='-highpass %f',
            mandatory=True,
        ),
        in_file=dict(
            argstr='%s',
            copyfile=False,
            extensions=None,
            mandatory=True,
            position=-1,
        ),
        lowpass=dict(
            argstr='-lowpass %f',
            mandatory=True,
        ),
        num_threads=dict(
            nohash=True,
            usedefault=True,
        ),
        out_file=dict(
            argstr='-prefix %s',
            extensions=None,
            name_source='in_file',
            name_template='%s_fourier',
        ),
        outputtype=dict(),
        retrend=dict(argstr='-retrend', ),
    )
    inputs = Fourier.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Fourier_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = Fourier.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..odf import MESD


def test_MESD_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        bgmask=dict(
            argstr='-bgmask %s',
            extensions=None,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fastmesd=dict(
            argstr='-fastmesd',
            requires=['mepointset'],
        ),
        in_file=dict(
            argstr='-inputfile %s',
            extensions=None,
            mandatory=True,
            position=1,
        ),
        inputdatatype=dict(argstr='-inputdatatype %s', ),
        inverter=dict(
            argstr='-filter %s',
            mandatory=True,
            position=2,
        ),
        inverter_param=dict(
            argstr='%f',
            mandatory=True,
            position=3,
            units='NA',
        ),
        mepointset=dict(
            argstr='-mepointset %d',
            units='NA',
        ),
        out_file=dict(
            argstr='> %s',
            extensions=None,
            genfile=True,
            position=-1,
        ),
        scheme_file=dict(
            argstr='-schemefile %s',
            extensions=None,
            mandatory=True,
        ),
    )
    inputs = MESD.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_MESD_outputs():
    output_map = dict(mesd_data=dict(extensions=None, ), )
    outputs = MESD.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..segmentation import BRAINSTalairach


def test_BRAINSTalairach_inputs():
    input_map = dict(
        AC=dict(
            argstr='--AC %s',
            sep=',',
        ),
        ACisIndex=dict(argstr='--ACisIndex ', ),
        IRP=dict(
            argstr='--IRP %s',
            sep=',',
        ),
        IRPisIndex=dict(argstr='--IRPisIndex ', ),
        PC=dict(
            argstr='--PC %s',
            sep=',',
        ),
        PCisIndex=dict(argstr='--PCisIndex ', ),
        SLA=dict(
            argstr='--SLA %s',
            sep=',',
        ),
        SLAisIndex=dict(argstr='--SLAisIndex ', ),
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputVolume=dict(
            argstr='--inputVolume %s',
            extensions=None,
        ),
        outputBox=dict(
            argstr='--outputBox %s',
            hash_files=False,
        ),
        outputGrid=dict(
            argstr='--outputGrid %s',
            hash_files=False,
        ),
    )
    inputs = BRAINSTalairach.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_BRAINSTalairach_outputs():
    output_map = dict(
        outputBox=dict(extensions=None, ),
        outputGrid=dict(extensions=None, ),
    )
    outputs = BRAINSTalairach.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..model import Level1Design


def test_Level1Design_inputs():
    input_map = dict(
        bases=dict(
            field='bases',
            mandatory=True,
        ),
        factor_info=dict(field='fact', ),
        flags=dict(),
        global_intensity_normalization=dict(field='global', ),
        interscan_interval=dict(
            field='timing.RT',
            mandatory=True,
        ),
        mask_image=dict(
            extensions=None,
            field='mask',
        ),
        mask_threshold=dict(usedefault=True, ),
        matlab_cmd=dict(),
        mfile=dict(usedefault=True, ),
        microtime_onset=dict(field='timing.fmri_t0', ),
        microtime_resolution=dict(field='timing.fmri_t', ),
        model_serial_correlations=dict(field='cvi', ),
        paths=dict(),
        session_info=dict(
            field='sess',
            mandatory=True,
        ),
        spm_mat_dir=dict(field='dir', ),
        timing_units=dict(
            field='timing.units',
            mandatory=True,
        ),
        use_mcr=dict(),
        use_v8struct=dict(
            min_ver='8',
            usedefault=True,
        ),
        volterra_expansion_order=dict(field='volt', ),
    )
    inputs = Level1Design.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Level1Design_outputs():
    output_map = dict(spm_mat_file=dict(extensions=None, ), )
    outputs = Level1Design.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""Miscellaneous utility functions
"""
from __future__ import (print_function, unicode_literals, division,
                        absolute_import)
import os
import sys
import gc
import errno
import select
import locale
import datetime
from subprocess import Popen, STDOUT, PIPE
from .filemanip import canonicalize_env, read_stream

from .. import logging

from builtins import range, object

iflogger = logging.getLogger('nipype.interface')


class Stream(object):
    """Function to capture stdout and stderr streams with timestamps

    stackoverflow.com/questions/4984549/merge-and-sync-stdout-and-stderr/5188359
    """

    def __init__(self, name, impl):
        self._name = name
        self._impl = impl
        self._buf = ''
        self._rows = []
        self._lastidx = 0
        self.default_encoding = locale.getdefaultlocale()[1] or 'UTF-8'

    def fileno(self):
        "Pass-through for file descriptor."
        return self._impl.fileno()

    def read(self, drain=0):
        "Read from the file descriptor. If 'drain' set, read until EOF."
        while self._read(drain) is not None:
            if not drain:
                break

    def _read(self, drain):
        "Read from the file descriptor"
        fd = self.fileno()
        buf = os.read(fd, 4096).decode(self.default_encoding)
        if not buf and not self._buf:
            return None
        if '\n' not in buf:
            if not drain:
                self._buf += buf
                return []

        # prepend any data previously read, then split into lines and format
        buf = self._buf + buf
        if '\n' in buf:
            tmp, rest = buf.rsplit('\n', 1)
        else:
            tmp = buf
            rest = None
        self._buf = rest
        now = datetime.datetime.now().isoformat()
        rows = tmp.split('\n')
        self._rows += [(now, '%s %s:%s' % (self._name, now, r), r)
                       for r in rows]
        for idx in range(self._lastidx, len(self._rows)):
            iflogger.info(self._rows[idx][1])
        self._lastidx = len(self._rows)


def run_command(runtime, output=None, timeout=0.01):
    """Run a command, read stdout and stderr, prefix with timestamp.

    The returned runtime contains a merged stdout+stderr log with timestamps
    """

    # Init variables
    cmdline = runtime.cmdline
    env = canonicalize_env(runtime.environ)

    errfile = None
    outfile = None
    stdout = PIPE
    stderr = PIPE

    if output == 'file':
        outfile = os.path.join(runtime.cwd, 'output.nipype')
        stdout = open(outfile, 'wb')  # t=='text'===default
        stderr = STDOUT
    elif output == 'file_split':
        outfile = os.path.join(runtime.cwd, 'stdout.nipype')
        stdout = open(outfile, 'wb')
        errfile = os.path.join(runtime.cwd, 'stderr.nipype')
        stderr = open(errfile, 'wb')
    elif output == 'file_stdout':
        outfile = os.path.join(runtime.cwd, 'stdout.nipype')
        stdout = open(outfile, 'wb')
    elif output == 'file_stderr':
        errfile = os.path.join(runtime.cwd, 'stderr.nipype')
        stderr = open(errfile, 'wb')

    proc = Popen(
        cmdline,
        stdout=stdout,
        stderr=stderr,
        shell=True,
        cwd=runtime.cwd,
        env=env,
        close_fds=(not sys.platform.startswith('win')),
    )

    result = {
        'stdout': [],
        'stderr': [],
        'merged': [],
    }

    if output == 'stream':
        streams = [
            Stream('stdout', proc.stdout),
            Stream('stderr', proc.stderr)
        ]

        def _process(drain=0):
            try:
                res = select.select(streams, [], [], timeout)
            except select.error as e:
                iflogger.info(e)
                if e[0] == errno.EINTR:
                    return
                else:
                    raise
            else:
                for stream in res[0]:
                    stream.read(drain)

        while proc.returncode is None:
            proc.poll()
            _process()

        _process(drain=1)

        # collect results, merge and return
        result = {}
        temp = []
        for stream in streams:
            rows = stream._rows
            temp += rows
            result[stream._name] = [r[2] for r in rows]
        temp.sort()
        result['merged'] = [r[1] for r in temp]

    if output.startswith('file'):
        proc.wait()
        if outfile is not None:
            stdout.flush()
            stdout.close()
            with open(outfile, 'rb') as ofh:
                stdoutstr = ofh.read()
            result['stdout'] = read_stream(stdoutstr, logger=iflogger)
            del stdoutstr

        if errfile is not None:
            stderr.flush()
            stderr.close()
            with open(errfile, 'rb') as efh:
                stderrstr = efh.read()
            result['stderr'] = read_stream(stderrstr, logger=iflogger)
            del stderrstr

        if output == 'file':
            result['merged'] = result['stdout']
            result['stdout'] = []
    else:
        stdout, stderr = proc.communicate()
        if output == 'allatonce':  # Discard stdout and stderr otherwise
            result['stdout'] = read_stream(stdout, logger=iflogger)
            result['stderr'] = read_stream(stderr, logger=iflogger)

    runtime.returncode = proc.returncode
    try:
        proc.terminate()  # Ensure we are done
    except OSError as error:
        # Python 2 raises when the process is already gone
        if error.errno != errno.ESRCH:
            raise

    # Dereference & force GC for a cleanup
    del proc
    del stdout
    del stderr
    gc.collect()

    runtime.stderr = '\n'.join(result['stderr'])
    runtime.stdout = '\n'.join(result['stdout'])
    runtime.merged = '\n'.join(result['merged'])
    return runtime



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..diffusion import dtiestim


def test_dtiestim_inputs():
    input_map = dict(
        B0=dict(
            argstr='--B0 %s',
            hash_files=False,
        ),
        B0_mask_output=dict(
            argstr='--B0_mask_output %s',
            hash_files=False,
        ),
        DTI_double=dict(argstr='--DTI_double ', ),
        args=dict(argstr='%s', ),
        bad_region_mask=dict(
            argstr='--bad_region_mask %s',
            extensions=None,
        ),
        brain_mask=dict(
            argstr='--brain_mask %s',
            extensions=None,
        ),
        correction=dict(argstr='--correction %s', ),
        defaultTensor=dict(
            argstr='--defaultTensor %s',
            sep=',',
        ),
        dwi_image=dict(
            argstr='--dwi_image %s',
            extensions=None,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        idwi=dict(
            argstr='--idwi %s',
            hash_files=False,
        ),
        method=dict(argstr='--method %s', ),
        shiftNeg=dict(argstr='--shiftNeg ', ),
        shiftNegCoeff=dict(argstr='--shiftNegCoeff %f', ),
        sigma=dict(argstr='--sigma %f', ),
        step=dict(argstr='--step %f', ),
        tensor_output=dict(
            argstr='--tensor_output %s',
            hash_files=False,
        ),
        threshold=dict(argstr='--threshold %d', ),
        verbose=dict(argstr='--verbose ', ),
        weight_iterations=dict(argstr='--weight_iterations %d', ),
    )
    inputs = dtiestim.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_dtiestim_outputs():
    output_map = dict(
        B0=dict(extensions=None, ),
        B0_mask_output=dict(extensions=None, ),
        idwi=dict(extensions=None, ),
        tensor_output=dict(extensions=None, ),
    )
    outputs = dtiestim.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import RobustRegister


def test_RobustRegister_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        auto_sens=dict(
            argstr='--satit',
            mandatory=True,
            xor=['outlier_sens'],
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        est_int_scale=dict(argstr='--iscale', ),
        force_double=dict(argstr='--doubleprec', ),
        force_float=dict(argstr='--floattype', ),
        half_source=dict(argstr='--halfmov %s', ),
        half_source_xfm=dict(argstr='--halfmovlta %s', ),
        half_targ=dict(argstr='--halfdst %s', ),
        half_targ_xfm=dict(argstr='--halfdstlta %s', ),
        half_weights=dict(argstr='--halfweights %s', ),
        high_iterations=dict(argstr='--highit %d', ),
        in_xfm_file=dict(
            argstr='--transform',
            extensions=None,
        ),
        init_orient=dict(argstr='--initorient', ),
        iteration_thresh=dict(argstr='--epsit %.3f', ),
        least_squares=dict(argstr='--leastsquares', ),
        mask_source=dict(
            argstr='--maskmov %s',
            extensions=None,
        ),
        mask_target=dict(
            argstr='--maskdst %s',
            extensions=None,
        ),
        max_iterations=dict(argstr='--maxit %d', ),
        no_init=dict(argstr='--noinit', ),
        no_multi=dict(argstr='--nomulti', ),
        out_reg_file=dict(
            argstr='--lta %s',
            usedefault=True,
        ),
        outlier_limit=dict(argstr='--wlimit %.3f', ),
        outlier_sens=dict(
            argstr='--sat %.4f',
            mandatory=True,
            xor=['auto_sens'],
        ),
        registered_file=dict(argstr='--warp %s', ),
        source_file=dict(
            argstr='--mov %s',
            extensions=None,
            mandatory=True,
        ),
        subjects_dir=dict(),
        subsample_thresh=dict(argstr='--subsample %d', ),
        target_file=dict(
            argstr='--dst %s',
            extensions=None,
            mandatory=True,
        ),
        trans_only=dict(argstr='--transonly', ),
        weights_file=dict(argstr='--weights %s', ),
        write_vo2vox=dict(argstr='--vox2vox', ),
    )
    inputs = RobustRegister.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_RobustRegister_outputs():
    output_map = dict(
        half_source=dict(extensions=None, ),
        half_source_xfm=dict(extensions=None, ),
        half_targ=dict(extensions=None, ),
        half_targ_xfm=dict(extensions=None, ),
        half_weights=dict(extensions=None, ),
        out_reg_file=dict(extensions=None, ),
        registered_file=dict(extensions=None, ),
        weights_file=dict(extensions=None, ),
    )
    outputs = RobustRegister.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..brainsfit import BRAINSFit


def test_BRAINSFit_inputs():
    input_map = dict(
        ROIAutoClosingSize=dict(argstr='--ROIAutoClosingSize %f', ),
        ROIAutoDilateSize=dict(argstr='--ROIAutoDilateSize %f', ),
        args=dict(argstr='%s', ),
        backgroundFillValue=dict(argstr='--backgroundFillValue %f', ),
        bsplineTransform=dict(
            argstr='--bsplineTransform %s',
            hash_files=False,
        ),
        costFunctionConvergenceFactor=dict(
            argstr='--costFunctionConvergenceFactor %f', ),
        costMetric=dict(argstr='--costMetric %s', ),
        debugLevel=dict(argstr='--debugLevel %d', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        failureExitCode=dict(argstr='--failureExitCode %d', ),
        fixedBinaryVolume=dict(
            argstr='--fixedBinaryVolume %s',
            extensions=None,
        ),
        fixedVolume=dict(
            argstr='--fixedVolume %s',
            extensions=None,
        ),
        fixedVolume2=dict(
            argstr='--fixedVolume2 %s',
            extensions=None,
        ),
        fixedVolumeTimeIndex=dict(argstr='--fixedVolumeTimeIndex %d', ),
        gui=dict(argstr='--gui ', ),
        histogramMatch=dict(argstr='--histogramMatch ', ),
        initialTransform=dict(
            argstr='--initialTransform %s',
            extensions=None,
        ),
        initializeRegistrationByCurrentGenericTransform=dict(
            argstr='--initializeRegistrationByCurrentGenericTransform ', ),
        initializeTransformMode=dict(argstr='--initializeTransformMode %s', ),
        interpolationMode=dict(argstr='--interpolationMode %s', ),
        linearTransform=dict(
            argstr='--linearTransform %s',
            hash_files=False,
        ),
        logFileReport=dict(
            argstr='--logFileReport %s',
            hash_files=False,
        ),
        maskInferiorCutOffFromCenter=dict(
            argstr='--maskInferiorCutOffFromCenter %f', ),
        maskProcessingMode=dict(argstr='--maskProcessingMode %s', ),
        maxBSplineDisplacement=dict(argstr='--maxBSplineDisplacement %f', ),
        maximumNumberOfCorrections=dict(
            argstr='--maximumNumberOfCorrections %d', ),
        maximumNumberOfEvaluations=dict(
            argstr='--maximumNumberOfEvaluations %d', ),
        maximumStepLength=dict(argstr='--maximumStepLength %f', ),
        medianFilterSize=dict(
            argstr='--medianFilterSize %s',
            sep=',',
        ),
        metricSamplingStrategy=dict(argstr='--metricSamplingStrategy %s', ),
        minimumStepLength=dict(
            argstr='--minimumStepLength %s',
            sep=',',
        ),
        movingBinaryVolume=dict(
            argstr='--movingBinaryVolume %s',
            extensions=None,
        ),
        movingVolume=dict(
            argstr='--movingVolume %s',
            extensions=None,
        ),
        movingVolume2=dict(
            argstr='--movingVolume2 %s',
            extensions=None,
        ),
        movingVolumeTimeIndex=dict(argstr='--movingVolumeTimeIndex %d', ),
        numberOfHistogramBins=dict(argstr='--numberOfHistogramBins %d', ),
        numberOfIterations=dict(
            argstr='--numberOfIterations %s',
            sep=',',
        ),
        numberOfMatchPoints=dict(argstr='--numberOfMatchPoints %d', ),
        numberOfSamples=dict(argstr='--numberOfSamples %d', ),
        numberOfThreads=dict(argstr='--numberOfThreads %d', ),
        outputFixedVolumeROI=dict(
            argstr='--outputFixedVolumeROI %s',
            hash_files=False,
        ),
        outputMovingVolumeROI=dict(
            argstr='--outputMovingVolumeROI %s',
            hash_files=False,
        ),
        outputTransform=dict(
            argstr='--outputTransform %s',
            hash_files=False,
        ),
        outputVolume=dict(
            argstr='--outputVolume %s',
            hash_files=False,
        ),
        outputVolumePixelType=dict(argstr='--outputVolumePixelType %s', ),
        projectedGradientTolerance=dict(
            argstr='--projectedGradientTolerance %f', ),
        promptUser=dict(argstr='--promptUser ', ),
        relaxationFactor=dict(argstr='--relaxationFactor %f', ),
        removeIntensityOutliers=dict(argstr='--removeIntensityOutliers %f', ),
        reproportionScale=dict(argstr='--reproportionScale %f', ),
        samplingPercentage=dict(argstr='--samplingPercentage %f', ),
        scaleOutputValues=dict(argstr='--scaleOutputValues ', ),
        skewScale=dict(argstr='--skewScale %f', ),
        splineGridSize=dict(
            argstr='--splineGridSize %s',
            sep=',',
        ),
        strippedOutputTransform=dict(
            argstr='--strippedOutputTransform %s',
            hash_files=False,
        ),
        transformType=dict(
            argstr='--transformType %s',
            sep=',',
        ),
        translationScale=dict(argstr='--translationScale %f', ),
        useAffine=dict(argstr='--useAffine ', ),
        useBSpline=dict(argstr='--useBSpline ', ),
        useComposite=dict(argstr='--useComposite ', ),
        useROIBSpline=dict(argstr='--useROIBSpline ', ),
        useRigid=dict(argstr='--useRigid ', ),
        useScaleSkewVersor3D=dict(argstr='--useScaleSkewVersor3D ', ),
        useScaleVersor3D=dict(argstr='--useScaleVersor3D ', ),
        useSyN=dict(argstr='--useSyN ', ),
        writeOutputTransformInFloat=dict(
            argstr='--writeOutputTransformInFloat ', ),
        writeTransformOnFailure=dict(argstr='--writeTransformOnFailure ', ),
    )
    inputs = BRAINSFit.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_BRAINSFit_outputs():
    output_map = dict(
        bsplineTransform=dict(extensions=None, ),
        linearTransform=dict(extensions=None, ),
        logFileReport=dict(extensions=None, ),
        outputFixedVolumeROI=dict(extensions=None, ),
        outputMovingVolumeROI=dict(extensions=None, ),
        outputTransform=dict(extensions=None, ),
        outputVolume=dict(extensions=None, ),
        strippedOutputTransform=dict(extensions=None, ),
    )
    outputs = BRAINSFit.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..featuredetection import CannySegmentationLevelSetImageFilter


def test_CannySegmentationLevelSetImageFilter_inputs():
    input_map = dict(
        advectionWeight=dict(argstr='--advectionWeight %f', ),
        args=dict(argstr='%s', ),
        cannyThreshold=dict(argstr='--cannyThreshold %f', ),
        cannyVariance=dict(argstr='--cannyVariance %f', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        initialModel=dict(
            argstr='--initialModel %s',
            extensions=None,
        ),
        initialModelIsovalue=dict(argstr='--initialModelIsovalue %f', ),
        inputVolume=dict(
            argstr='--inputVolume %s',
            extensions=None,
        ),
        maxIterations=dict(argstr='--maxIterations %d', ),
        outputSpeedVolume=dict(
            argstr='--outputSpeedVolume %s',
            hash_files=False,
        ),
        outputVolume=dict(
            argstr='--outputVolume %s',
            hash_files=False,
        ),
    )
    inputs = CannySegmentationLevelSetImageFilter.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_CannySegmentationLevelSetImageFilter_outputs():
    output_map = dict(
        outputSpeedVolume=dict(extensions=None, ),
        outputVolume=dict(extensions=None, ),
    )
    outputs = CannySegmentationLevelSetImageFilter.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..reg import RegF3D


def test_RegF3D_inputs():
    input_map = dict(
        aff_file=dict(
            argstr='-aff %s',
            extensions=None,
        ),
        amc_flag=dict(argstr='-amc', ),
        args=dict(argstr='%s', ),
        be_val=dict(argstr='-be %f', ),
        cpp_file=dict(
            argstr='-cpp %s',
            extensions=None,
            name_source=['flo_file'],
            name_template='%s_cpp.nii.gz',
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fbn2_val=dict(argstr='-fbn %d %d', ),
        fbn_val=dict(argstr='--fbn %d', ),
        flo_file=dict(
            argstr='-flo %s',
            extensions=None,
            mandatory=True,
        ),
        flo_smooth_val=dict(argstr='-smooF %f', ),
        flwth2_thr_val=dict(argstr='-fLwTh %d %f', ),
        flwth_thr_val=dict(argstr='--fLwTh %f', ),
        fmask_file=dict(
            argstr='-fmask %s',
            extensions=None,
        ),
        fupth2_thr_val=dict(argstr='-fUpTh %d %f', ),
        fupth_thr_val=dict(argstr='--fUpTh %f', ),
        incpp_file=dict(
            argstr='-incpp %s',
            extensions=None,
        ),
        jl_val=dict(argstr='-jl %f', ),
        kld2_flag=dict(argstr='-kld %d', ),
        kld_flag=dict(argstr='--kld', ),
        le_val=dict(argstr='-le %f', ),
        ln_val=dict(argstr='-ln %d', ),
        lncc2_val=dict(argstr='-lncc %d %f', ),
        lncc_val=dict(argstr='--lncc %f', ),
        lp_val=dict(argstr='-lp %d', ),
        maxit_val=dict(argstr='-maxit %d', ),
        nmi_flag=dict(argstr='--nmi', ),
        no_app_jl_flag=dict(argstr='-noAppJL', ),
        noconj_flag=dict(argstr='-noConj', ),
        nopy_flag=dict(argstr='-nopy', ),
        nox_flag=dict(argstr='-nox', ),
        noy_flag=dict(argstr='-noy', ),
        noz_flag=dict(argstr='-noz', ),
        omp_core_val=dict(
            argstr='-omp %i',
            usedefault=True,
        ),
        pad_val=dict(argstr='-pad %f', ),
        pert_val=dict(argstr='-pert %d', ),
        rbn2_val=dict(argstr='-rbn %d %d', ),
        rbn_val=dict(argstr='--rbn %d', ),
        ref_file=dict(
            argstr='-ref %s',
            extensions=None,
            mandatory=True,
        ),
        ref_smooth_val=dict(argstr='-smooR %f', ),
        res_file=dict(
            argstr='-res %s',
            extensions=None,
            name_source=['flo_file'],
            name_template='%s_res.nii.gz',
        ),
        rlwth2_thr_val=dict(argstr='-rLwTh %d %f', ),
        rlwth_thr_val=dict(argstr='--rLwTh %f', ),
        rmask_file=dict(
            argstr='-rmask %s',
            extensions=None,
        ),
        rupth2_thr_val=dict(argstr='-rUpTh %d %f', ),
        rupth_thr_val=dict(argstr='--rUpTh %f', ),
        smooth_grad_val=dict(argstr='-smoothGrad %f', ),
        ssd2_flag=dict(argstr='-ssd %d', ),
        ssd_flag=dict(argstr='--ssd', ),
        sx_val=dict(argstr='-sx %f', ),
        sy_val=dict(argstr='-sy %f', ),
        sz_val=dict(argstr='-sz %f', ),
        vel_flag=dict(argstr='-vel', ),
        verbosity_off_flag=dict(argstr='-voff', ),
    )
    inputs = RegF3D.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_RegF3D_outputs():
    output_map = dict(
        avg_output=dict(),
        cpp_file=dict(extensions=None, ),
        invcpp_file=dict(extensions=None, ),
        invres_file=dict(extensions=None, ),
        res_file=dict(extensions=None, ),
    )
    outputs = RegF3D.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import MotionOutliers


def test_MotionOutliers_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        dummy=dict(argstr='--dummy=%d', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='-i %s',
            extensions=None,
            mandatory=True,
        ),
        mask=dict(
            argstr='-m %s',
            extensions=None,
        ),
        metric=dict(argstr='--%s', ),
        no_motion_correction=dict(argstr='--nomoco', ),
        out_file=dict(
            argstr='-o %s',
            extensions=None,
            hash_files=False,
            keep_extension=True,
            name_source='in_file',
            name_template='%s_outliers.txt',
        ),
        out_metric_plot=dict(
            argstr='-p %s',
            extensions=None,
            hash_files=False,
            keep_extension=True,
            name_source='in_file',
            name_template='%s_metrics.png',
        ),
        out_metric_values=dict(
            argstr='-s %s',
            extensions=None,
            hash_files=False,
            keep_extension=True,
            name_source='in_file',
            name_template='%s_metrics.txt',
        ),
        output_type=dict(),
        threshold=dict(argstr='--thresh=%g', ),
    )
    inputs = MotionOutliers.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_MotionOutliers_outputs():
    output_map = dict(
        out_file=dict(extensions=None, ),
        out_metric_plot=dict(extensions=None, ),
        out_metric_values=dict(extensions=None, ),
    )
    outputs = MotionOutliers.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..brainsuite import Bse


def test_Bse_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        diffusionConstant=dict(
            argstr='-d %f',
            usedefault=True,
        ),
        diffusionIterations=dict(
            argstr='-n %d',
            usedefault=True,
        ),
        dilateFinalMask=dict(
            argstr='-p',
            usedefault=True,
        ),
        edgeDetectionConstant=dict(
            argstr='-s %f',
            usedefault=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputMRIFile=dict(
            argstr='-i %s',
            extensions=None,
            mandatory=True,
        ),
        noRotate=dict(argstr='--norotate', ),
        outputCortexFile=dict(
            argstr='--cortex %s',
            extensions=None,
            hash_files=False,
        ),
        outputDetailedBrainMask=dict(
            argstr='--hires %s',
            extensions=None,
            hash_files=False,
        ),
        outputDiffusionFilter=dict(
            argstr='--adf %s',
            extensions=None,
            hash_files=False,
        ),
        outputEdgeMap=dict(
            argstr='--edge %s',
            extensions=None,
            hash_files=False,
        ),
        outputMRIVolume=dict(
            argstr='-o %s',
            extensions=None,
            genfile=True,
            hash_files=False,
        ),
        outputMaskFile=dict(
            argstr='--mask %s',
            extensions=None,
            genfile=True,
            hash_files=False,
        ),
        radius=dict(
            argstr='-r %f',
            usedefault=True,
        ),
        timer=dict(argstr='--timer', ),
        trim=dict(
            argstr='--trim',
            usedefault=True,
        ),
        verbosityLevel=dict(
            argstr='-v %f',
            usedefault=True,
        ),
    )
    inputs = Bse.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Bse_outputs():
    output_map = dict(
        outputCortexFile=dict(extensions=None, ),
        outputDetailedBrainMask=dict(extensions=None, ),
        outputDiffusionFilter=dict(extensions=None, ),
        outputEdgeMap=dict(extensions=None, ),
        outputMRIVolume=dict(extensions=None, ),
        outputMaskFile=dict(extensions=None, ),
    )
    outputs = Bse.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..possum import B0Calc


def test_B0Calc_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        chi_air=dict(
            argstr='--chi0=%e',
            usedefault=True,
        ),
        compute_xyz=dict(
            argstr='--xyz',
            usedefault=True,
        ),
        delta=dict(
            argstr='-d %e',
            usedefault=True,
        ),
        directconv=dict(
            argstr='--directconv',
            usedefault=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        extendboundary=dict(
            argstr='--extendboundary=%0.2f',
            usedefault=True,
        ),
        in_file=dict(
            argstr='-i %s',
            extensions=None,
            mandatory=True,
            position=0,
        ),
        out_file=dict(
            argstr='-o %s',
            extensions=None,
            name_source=['in_file'],
            name_template='%s_b0field',
            output_name='out_file',
            position=1,
        ),
        output_type=dict(),
        x_b0=dict(
            argstr='--b0x=%0.2f',
            usedefault=True,
            xor=['xyz_b0'],
        ),
        x_grad=dict(
            argstr='--gx=%0.4f',
            usedefault=True,
        ),
        xyz_b0=dict(
            argstr='--b0x=%0.2f --b0y=%0.2f --b0=%0.2f',
            xor=['x_b0', 'y_b0', 'z_b0'],
        ),
        y_b0=dict(
            argstr='--b0y=%0.2f',
            usedefault=True,
            xor=['xyz_b0'],
        ),
        y_grad=dict(
            argstr='--gy=%0.4f',
            usedefault=True,
        ),
        z_b0=dict(
            argstr='--b0=%0.2f',
            usedefault=True,
            xor=['xyz_b0'],
        ),
        z_grad=dict(
            argstr='--gz=%0.4f',
            usedefault=True,
        ),
    )
    inputs = B0Calc.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_B0Calc_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = B0Calc.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# coding: utf-8
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:

from builtins import range
import os

import pytest
from nipype.testing import example_data

import numpy as np
import nibabel as nb
import nipype.testing as nit

from nipype.algorithms.misc import normalize_tpms
from nipype.utils import NUMPY_MMAP


def test_normalize_tpms(tmpdir):

    in_mask = example_data('tpms_msk.nii.gz')
    mskdata = nb.load(in_mask, mmap=NUMPY_MMAP).get_data()
    mskdata[mskdata > 0.0] = 1.0

    mapdata = []
    in_files = []
    out_files = []

    for i in range(3):
        mapname = example_data('tpm_%02d.nii.gz' % i)
        filename = tmpdir.join('modtpm_%02d.nii.gz' % i).strpath
        out_files.append(tmpdir.join('normtpm_%02d.nii.gz' % i).strpath)

        im = nb.load(mapname, mmap=NUMPY_MMAP)
        data = im.get_data()
        mapdata.append(data.copy())

        nb.Nifti1Image(2.0 * (data * mskdata), im.affine,
                       im.header).to_filename(filename)
        in_files.append(filename)

    normalize_tpms(in_files, in_mask, out_files=out_files)

    sumdata = np.zeros_like(mskdata)

    for i, tstfname in enumerate(out_files):
        normdata = nb.load(tstfname, mmap=NUMPY_MMAP).get_data()
        sumdata += normdata
        assert np.all(normdata[mskdata == 0] == 0)
        assert np.allclose(normdata, mapdata[i])

    assert np.allclose(sumdata[sumdata > 0.0], 1.0)



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import print_function, unicode_literals

import pytest
from ..base import EngineBase
from ....interfaces import base as nib
from ....interfaces import utility as niu
from ... import engine as pe


class InputSpec(nib.TraitedSpec):
    input1 = nib.traits.Int(desc='a random int')
    input2 = nib.traits.Int(desc='a random int')
    input_file = nib.File(desc='Random File')


class OutputSpec(nib.TraitedSpec):
    output1 = nib.traits.List(nib.traits.Int, desc='outputs')


class EngineTestInterface(nib.SimpleInterface):
    input_spec = InputSpec
    output_spec = OutputSpec

    def _run_interface(self, runtime):
        runtime.returncode = 0
        self._results['output1'] = [1, self.inputs.input1]
        return runtime


@pytest.mark.parametrize(
    'name', ['valid1', 'valid_node', 'valid-node', 'ValidNode0'])
def test_create(name):
    base = EngineBase(name=name)
    assert base.name == name


@pytest.mark.parametrize(
    'name', ['invalid*1', 'invalid.1', 'invalid@', 'in/valid', None])
def test_create_invalid(name):
    with pytest.raises(ValueError):
        EngineBase(name=name)


def test_hierarchy():
    base = EngineBase(name='nodename')
    base._hierarchy = 'some.history.behind'

    assert base.name == 'nodename'
    assert base.fullname == 'some.history.behind.nodename'


def test_clone():
    base = EngineBase(name='nodename')
    base2 = base.clone('newnodename')

    assert (base.base_dir == base2.base_dir and
            base.config == base2.config and
            base2.name == 'newnodename')

    with pytest.raises(ValueError):
        base.clone('nodename')

def test_clone_node_iterables(tmpdir):
    tmpdir.chdir()

    def addstr(string):
        return ('%s + 2' % string)

    subject_list = ['sub-001', 'sub-002']
    inputnode = pe.Node(niu.IdentityInterface(fields=['subject']),
                            name='inputnode')
    inputnode.iterables = [('subject', subject_list)]

    node_1 = pe.Node(niu.Function(input_names='string',
                                  output_names='string',
                                  function=addstr), name='node_1')
    node_2 = node_1.clone('node_2')

    workflow = pe.Workflow(name='iter_clone_wf')
    workflow.connect([(inputnode, node_1, [('subject', 'string')]),
                      (node_1, node_2, [('string', 'string')])])
    workflow.run()



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..tracking import Tractography


def test_Tractography_inputs():
    input_map = dict(
        act_file=dict(
            argstr='-act %s',
            extensions=None,
        ),
        algorithm=dict(
            argstr='-algorithm %s',
            usedefault=True,
        ),
        angle=dict(argstr='-angle %f', ),
        args=dict(argstr='%s', ),
        backtrack=dict(argstr='-backtrack', ),
        bval_scale=dict(argstr='-bvalue_scaling %s', ),
        crop_at_gmwmi=dict(argstr='-crop_at_gmwmi', ),
        cutoff=dict(argstr='-cutoff %f', ),
        cutoff_init=dict(argstr='-initcutoff %f', ),
        downsample=dict(argstr='-downsample %f', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        grad_file=dict(
            argstr='-grad %s',
            extensions=None,
            xor=['grad_fsl'],
        ),
        grad_fsl=dict(
            argstr='-fslgrad %s %s',
            xor=['grad_file'],
        ),
        in_bval=dict(extensions=None, ),
        in_bvec=dict(
            argstr='-fslgrad %s %s',
            extensions=None,
        ),
        in_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        init_dir=dict(argstr='-initdirection %f,%f,%f', ),
        max_length=dict(argstr='-maxlength %f', ),
        max_seed_attempts=dict(argstr='-max_seed_attempts %d', ),
        max_tracks=dict(argstr='-maxnum %d', ),
        min_length=dict(argstr='-minlength %f', ),
        n_samples=dict(
            argstr='-samples %d',
            usedefault=True,
        ),
        n_tracks=dict(
            argstr='-number %d',
            max_ver='0.4',
        ),
        n_trials=dict(argstr='-trials %d', ),
        noprecompt=dict(argstr='-noprecomputed', ),
        nthreads=dict(
            argstr='-nthreads %d',
            nohash=True,
        ),
        out_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-1,
            usedefault=True,
        ),
        out_seeds=dict(
            argstr='-output_seeds %s',
            extensions=None,
            usedefault=True,
        ),
        power=dict(argstr='-power %d', ),
        roi_excl=dict(argstr='-exclude %s', ),
        roi_incl=dict(argstr='-include %s', ),
        roi_mask=dict(argstr='-mask %s', ),
        seed_dynamic=dict(
            argstr='-seed_dynamic %s',
            extensions=None,
        ),
        seed_gmwmi=dict(
            argstr='-seed_gmwmi %s',
            extensions=None,
            requires=['act_file'],
        ),
        seed_grid_voxel=dict(
            argstr='-seed_grid_per_voxel %s %d',
            xor=['seed_image', 'seed_rnd_voxel'],
        ),
        seed_image=dict(
            argstr='-seed_image %s',
            extensions=None,
        ),
        seed_rejection=dict(
            argstr='-seed_rejection %s',
            extensions=None,
        ),
        seed_rnd_voxel=dict(
            argstr='-seed_random_per_voxel %s %d',
            xor=['seed_image', 'seed_grid_voxel'],
        ),
        seed_sphere=dict(argstr='-seed_sphere %f,%f,%f,%f', ),
        select=dict(
            argstr='-select %d',
            min_ver='3',
        ),
        sph_trait=dict(argstr='%f,%f,%f,%f', ),
        step_size=dict(argstr='-step %f', ),
        stop=dict(argstr='-stop', ),
        unidirectional=dict(argstr='-unidirectional', ),
        use_rk4=dict(argstr='-rk4', ),
    )
    inputs = Tractography.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Tractography_outputs():
    output_map = dict(
        out_file=dict(extensions=None, ),
        out_seeds=dict(extensions=None, ),
    )
    outputs = Tractography.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""The afni module provides classes for interfacing with the `AFNI
<http://afni.nimh.nih.gov/afni>`_ command line tools.

Top-level namespace for afni.
"""

from .base import Info
from .preprocess import (
    AlignEpiAnatPy, Allineate, Automask, AutoTcorrelate, AutoTLRC, Bandpass,
    BlurInMask, BlurToFWHM, ClipLevel, DegreeCentrality, Despike, Detrend, ECM,
    Fim, Fourier, Hist, LFCD, Maskave, Means, OutlierCount, QualityIndex,
    ROIStats, Retroicor, Seg, SkullStrip, TCorr1D, TCorrMap, TCorrelate, TNorm,
    TProject, TShift, TSmooth, Volreg, Warp, QwarpPlusMinus, Qwarp)
from .svm import (SVMTest, SVMTrain)
from .utils import (
    ABoverlap, AFNItoNIFTI, Autobox, Axialize, BrickStat, Bucket, Calc, Cat,
    CatMatvec, CenterMass, ConvertDset, Copy, Dot, Edge3, Eval, FWHMx,
    LocalBistat, Localstat, MaskTool, Merge, Notes, NwarpApply, NwarpAdjust,
    NwarpCat, OneDToolPy, Refit, ReHo, Resample, TCat, TCatSubBrick, TStat,
    To3D, Unifize, Undump, ZCutUp, GCOR, Zcat, Zeropad)
from .model import (Deconvolve, Remlfit, Synthesize)



# -*- coding: utf-8 -*-
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
import os

from ...utils.filemanip import split_filename
from ..base import (traits, TraitedSpec, File, CommandLine,
                    CommandLineInputSpec, isdefined)


class ConmatInputSpec(CommandLineInputSpec):
    in_file = File(
        exists=True,
        argstr='-inputfile %s',
        mandatory=True,
        desc='Streamlines as generated by the Track interface')

    target_file = File(
        exists=True,
        argstr='-targetfile %s',
        mandatory=True,
        desc=
        'An image containing targets, as used in ProcStreamlines interface.')

    scalar_file = File(
        exists=True,
        argstr='-scalarfile %s',
        desc=('Optional scalar file for computing tract-based statistics. '
              'Must be in the same space as the target file.'),
        requires=['tract_stat'])

    targetname_file = File(
        exists=True,
        argstr='-targetnamefile %s',
        desc=
        ('Optional names of targets. This file should contain one entry per line, '
         'with the target intensity followed by the name, separated by white space. '
         'For example: '
         '  1  some_brain_region '
         '  2     some_other_region '
         'These names will be used in the output. The names themselves should not '
         'contain spaces or commas. The labels may be in any order but the output '
         'matrices will be ordered by label intensity.'))

    tract_stat = traits.Enum(
        "mean",
        "min",
        "max",
        "sum",
        "median",
        "var",
        argstr='-tractstat %s',
        units='NA',
        desc=("Tract statistic to use. See TractStats for other options."),
        requires=['scalar_file'],
        xor=['tract_prop'])

    tract_prop = traits.Enum(
        "length",
        "endpointsep",
        argstr='-tractstat %s',
        units='NA',
        xor=['tract_stat'],
        desc=('Tract property average to compute in the connectivity matrix. '
              'See TractStats for details.'))

    output_root = File(
        argstr='-outputroot %s',
        genfile=True,
        desc=('filename root prepended onto the names of the output files. '
              'The extension will be determined from the input.'))


class ConmatOutputSpec(TraitedSpec):
    conmat_sc = File(exists=True, desc='Connectivity matrix in CSV file.')
    conmat_ts = File(desc='Tract statistics in CSV file.')


class Conmat(CommandLine):
    """
    Creates  a  connectivity  matrix  using a 3D label image (the target image)
    and a set of streamlines. The connectivity matrix records how many stream-
    lines connect each pair of targets, and optionally the mean tractwise
    statistic (eg tract-averaged FA, or length).

    The output is a comma separated variable file or files. The first row of
    the output matrix is label names. Label names may be defined by the user,
    otherwise  they  are assigned based on label intensity.

    Starting  from the seed point, we move along the streamline until we find
    a point in a labeled region. This is done in both directions from the seed
    point. Streamlines are counted if they connect two target regions, one on
    either side of the seed point. Only the labeled region closest to the seed
    is counted, for example if the  input contains two streamlines: ::

         1: A-----B------SEED---C
         2: A--------SEED-----------

    then the output would be ::

         A,B,C
         0,0,0
         0,0,1
         0,1,0

    There  are  zero  connections  to A because in streamline 1, the connection
    to B is closer to the seed than the connection to A, and in streamline 2
    there is no region reached in the other direction.

    The connected target regions can have the same label, as long as the seed
    point is outside of the labeled region and both ends connect to the same
    label (which may  be in different locations). Therefore this is allowed: ::

         A------SEED-------A

    Such fibers will add to the diagonal elements of the matrix. To remove
    these entries, run procstreamlines with -endpointfile before running conmat.

    If the seed point is inside a labled region, it counts as one end of the
    connection.  So ::

         ----[SEED inside A]---------B

    counts as a connection between A and B, while ::

         C----[SEED inside A]---------B

    counts as a connection between A and C, because C is closer to the seed point.

    In all cases, distance to the seed point is defined along the streamline path.

    Example 1
    ---------
    To create a standard connectivity matrix based on streamline counts.

    >>> import nipype.interfaces.camino as cam
    >>> conmat = cam.Conmat()
    >>> conmat.inputs.in_file = 'tracts.Bdouble'
    >>> conmat.inputs.target_file = 'atlas.nii.gz'
    >>> conmat.run()        # doctest: +SKIP

    Example 1
    ---------
    To create a standard connectivity matrix and mean tractwise FA statistics.

    >>> import nipype.interfaces.camino as cam
    >>> conmat = cam.Conmat()
    >>> conmat.inputs.in_file = 'tracts.Bdouble'
    >>> conmat.inputs.target_file = 'atlas.nii.gz'
    >>> conmat.inputs.scalar_file = 'fa.nii.gz'
    >>> conmat.tract_stat         = 'mean'
    >>> conmat.run()        # doctest: +SKIP
    """
    _cmd = 'conmat'
    input_spec = ConmatInputSpec
    output_spec = ConmatOutputSpec

    def _list_outputs(self):
        outputs = self.output_spec().get()
        output_root = self._gen_outputroot()
        outputs['conmat_sc'] = os.path.abspath(output_root + "sc.csv")
        outputs['conmat_ts'] = os.path.abspath(output_root + "ts.csv")
        return outputs

    def _gen_outfilename(self):
        return self._gen_outputroot()

    def _gen_outputroot(self):
        output_root = self.inputs.output_root
        if not isdefined(output_root):
            output_root = self._gen_filename('output_root')
        return output_root

    def _gen_filename(self, name):
        if name == 'output_root':
            _, filename, _ = split_filename(self.inputs.in_file)
            filename = filename + "_"
        return filename



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..dcmstack import DcmStack


def test_DcmStack_inputs():
    input_map = dict(
        dicom_files=dict(mandatory=True, ),
        embed_meta=dict(),
        exclude_regexes=dict(),
        force_read=dict(usedefault=True, ),
        include_regexes=dict(),
        out_ext=dict(usedefault=True, ),
        out_format=dict(),
        out_path=dict(),
    )
    inputs = DcmStack.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_DcmStack_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = DcmStack.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import CARegister


def test_CARegister_inputs():
    input_map = dict(
        A=dict(argstr='-A %d', ),
        align=dict(argstr='-align-%s', ),
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='%s',
            extensions=None,
            mandatory=True,
            position=-3,
        ),
        invert_and_save=dict(
            argstr='-invert-and-save',
            position=-4,
        ),
        l_files=dict(argstr='-l %s', ),
        levels=dict(argstr='-levels %d', ),
        mask=dict(
            argstr='-mask %s',
            extensions=None,
        ),
        no_big_ventricles=dict(argstr='-nobigventricles', ),
        num_threads=dict(),
        out_file=dict(
            argstr='%s',
            extensions=None,
            genfile=True,
            position=-1,
        ),
        subjects_dir=dict(),
        template=dict(
            argstr='%s',
            extensions=None,
            position=-2,
        ),
        transform=dict(
            argstr='-T %s',
            extensions=None,
        ),
    )
    inputs = CARegister.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_CARegister_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = CARegister.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..surface import ModelMaker


def test_ModelMaker_inputs():
    input_map = dict(
        InputVolume=dict(
            argstr='%s',
            extensions=None,
            position=-1,
        ),
        args=dict(argstr='%s', ),
        color=dict(
            argstr='--color %s',
            extensions=None,
        ),
        debug=dict(argstr='--debug ', ),
        decimate=dict(argstr='--decimate %f', ),
        end=dict(argstr='--end %d', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        filtertype=dict(argstr='--filtertype %s', ),
        generateAll=dict(argstr='--generateAll ', ),
        jointsmooth=dict(argstr='--jointsmooth ', ),
        labels=dict(
            argstr='--labels %s',
            sep=',',
        ),
        modelSceneFile=dict(
            argstr='--modelSceneFile %s...',
            hash_files=False,
        ),
        name=dict(argstr='--name %s', ),
        pad=dict(argstr='--pad ', ),
        pointnormals=dict(argstr='--pointnormals ', ),
        saveIntermediateModels=dict(argstr='--saveIntermediateModels ', ),
        skipUnNamed=dict(argstr='--skipUnNamed ', ),
        smooth=dict(argstr='--smooth %d', ),
        splitnormals=dict(argstr='--splitnormals ', ),
        start=dict(argstr='--start %d', ),
    )
    inputs = ModelMaker.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_ModelMaker_outputs():
    output_map = dict(modelSceneFile=dict(), )
    outputs = ModelMaker.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import NewSegment


def test_NewSegment_inputs():
    input_map = dict(
        affine_regularization=dict(field='warp.affreg', ),
        channel_files=dict(
            copyfile=False,
            field='channel',
            mandatory=True,
        ),
        channel_info=dict(field='channel', ),
        matlab_cmd=dict(),
        mfile=dict(usedefault=True, ),
        paths=dict(),
        sampling_distance=dict(field='warp.samp', ),
        tissues=dict(field='tissue', ),
        use_mcr=dict(),
        use_v8struct=dict(
            min_ver='8',
            usedefault=True,
        ),
        warping_regularization=dict(field='warp.reg', ),
        write_deformation_fields=dict(field='warp.write', ),
    )
    inputs = NewSegment.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_NewSegment_outputs():
    output_map = dict(
        bias_corrected_images=dict(),
        bias_field_images=dict(),
        dartel_input_images=dict(),
        forward_deformation_field=dict(),
        inverse_deformation_field=dict(),
        modulated_class_images=dict(),
        native_class_images=dict(),
        normalized_class_images=dict(),
        transformation_mat=dict(),
    )
    outputs = NewSegment.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..converters import DWISimpleCompare


def test_DWISimpleCompare_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        checkDWIData=dict(argstr='--checkDWIData ', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputVolume1=dict(
            argstr='--inputVolume1 %s',
            extensions=None,
        ),
        inputVolume2=dict(
            argstr='--inputVolume2 %s',
            extensions=None,
        ),
    )
    inputs = DWISimpleCompare.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_DWISimpleCompare_outputs():
    output_map = dict()
    outputs = DWISimpleCompare.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import Similarity


def test_Similarity_inputs():
    input_map = dict(
        mask1=dict(extensions=None, ),
        mask2=dict(extensions=None, ),
        metric=dict(usedefault=True, ),
        volume1=dict(
            extensions=None,
            mandatory=True,
        ),
        volume2=dict(
            extensions=None,
            mandatory=True,
        ),
    )
    inputs = Similarity.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Similarity_outputs():
    output_map = dict(similarity=dict(), )
    outputs = Similarity.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..model import Randomise


def test_Randomise_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        base_name=dict(
            argstr='-o "%s"',
            position=1,
            usedefault=True,
        ),
        c_thresh=dict(argstr='-c %.1f', ),
        cm_thresh=dict(argstr='-C %.1f', ),
        demean=dict(argstr='-D', ),
        design_mat=dict(
            argstr='-d %s',
            extensions=None,
            position=2,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        f_c_thresh=dict(argstr='-F %.2f', ),
        f_cm_thresh=dict(argstr='-S %.2f', ),
        f_only=dict(argstr='--f_only', ),
        fcon=dict(
            argstr='-f %s',
            extensions=None,
        ),
        in_file=dict(
            argstr='-i %s',
            extensions=None,
            mandatory=True,
            position=0,
        ),
        mask=dict(
            argstr='-m %s',
            extensions=None,
        ),
        num_perm=dict(argstr='-n %d', ),
        one_sample_group_mean=dict(argstr='-1', ),
        output_type=dict(),
        p_vec_n_dist_files=dict(argstr='-P', ),
        raw_stats_imgs=dict(argstr='-R', ),
        seed=dict(argstr='--seed=%d', ),
        show_info_parallel_mode=dict(argstr='-Q', ),
        show_total_perms=dict(argstr='-q', ),
        tcon=dict(
            argstr='-t %s',
            extensions=None,
            position=3,
        ),
        tfce=dict(argstr='-T', ),
        tfce2D=dict(argstr='--T2', ),
        tfce_C=dict(argstr='--tfce_C=%.2f', ),
        tfce_E=dict(argstr='--tfce_E=%.2f', ),
        tfce_H=dict(argstr='--tfce_H=%.2f', ),
        var_smooth=dict(argstr='-v %d', ),
        vox_p_values=dict(argstr='-x', ),
        x_block_labels=dict(
            argstr='-e %s',
            extensions=None,
        ),
    )
    inputs = Randomise.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Randomise_outputs():
    output_map = dict(
        f_corrected_p_files=dict(),
        f_p_files=dict(),
        fstat_files=dict(),
        t_corrected_p_files=dict(),
        t_p_files=dict(),
        tstat_files=dict(),
    )
    outputs = Randomise.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..dti import VecReg


def test_VecReg_inputs():
    input_map = dict(
        affine_mat=dict(
            argstr='-t %s',
            extensions=None,
        ),
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='-i %s',
            extensions=None,
            mandatory=True,
        ),
        interpolation=dict(argstr='--interp=%s', ),
        mask=dict(
            argstr='-m %s',
            extensions=None,
        ),
        out_file=dict(
            argstr='-o %s',
            extensions=None,
            genfile=True,
            hash_files=False,
        ),
        output_type=dict(),
        ref_mask=dict(
            argstr='--refmask=%s',
            extensions=None,
        ),
        ref_vol=dict(
            argstr='-r %s',
            extensions=None,
            mandatory=True,
        ),
        rotation_mat=dict(
            argstr='--rotmat=%s',
            extensions=None,
        ),
        rotation_warp=dict(
            argstr='--rotwarp=%s',
            extensions=None,
        ),
        warp_field=dict(
            argstr='-w %s',
            extensions=None,
        ),
    )
    inputs = VecReg.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_VecReg_outputs():
    output_map = dict(out_file=dict(extensions=None, ), )
    outputs = VecReg.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..model import TwoSampleTTestDesign


def test_TwoSampleTTestDesign_inputs():
    input_map = dict(
        covariates=dict(field='cov', ),
        dependent=dict(field='des.t2.dept', ),
        explicit_mask_file=dict(
            extensions=None,
            field='masking.em',
        ),
        global_calc_mean=dict(
            field='globalc.g_mean',
            xor=['global_calc_omit', 'global_calc_values'],
        ),
        global_calc_omit=dict(
            field='globalc.g_omit',
            xor=['global_calc_mean', 'global_calc_values'],
        ),
        global_calc_values=dict(
            field='globalc.g_user.global_uval',
            xor=['global_calc_mean', 'global_calc_omit'],
        ),
        global_normalization=dict(field='globalm.glonorm', ),
        group1_files=dict(
            field='des.t2.scans1',
            mandatory=True,
        ),
        group2_files=dict(
            field='des.t2.scans2',
            mandatory=True,
        ),
        matlab_cmd=dict(),
        mfile=dict(usedefault=True, ),
        no_grand_mean_scaling=dict(field='globalm.gmsca.gmsca_no', ),
        paths=dict(),
        spm_mat_dir=dict(field='dir', ),
        threshold_mask_absolute=dict(
            field='masking.tm.tma.athresh',
            xor=['threshold_mask_none', 'threshold_mask_relative'],
        ),
        threshold_mask_none=dict(
            field='masking.tm.tm_none',
            xor=['threshold_mask_absolute', 'threshold_mask_relative'],
        ),
        threshold_mask_relative=dict(
            field='masking.tm.tmr.rthresh',
            xor=['threshold_mask_absolute', 'threshold_mask_none'],
        ),
        unequal_variance=dict(field='des.t2.variance', ),
        use_implicit_threshold=dict(field='masking.im', ),
        use_mcr=dict(),
        use_v8struct=dict(
            min_ver='8',
            usedefault=True,
        ),
    )
    inputs = TwoSampleTTestDesign.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_TwoSampleTTestDesign_outputs():
    output_map = dict(spm_mat_file=dict(extensions=None, ), )
    outputs = TwoSampleTTestDesign.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# -*- coding: utf-8 -*-
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import open, object, str, bytes

# Py2 compat: http://python-future.org/compatible_idioms.html#collections-counter-and-ordereddict
from future import standard_library
standard_library.install_aliases()
from collections import OrderedDict

from copy import deepcopy
from pickle import dumps
import os
import getpass
import platform
from uuid import uuid1
import simplejson as json

import numpy as np
import prov.model as pm

from .. import get_info, logging, __version__
from .filemanip import (md5, hashlib, hash_infile)

logger = logging.getLogger('nipype.utils')
foaf = pm.Namespace("foaf", "http://xmlns.com/foaf/0.1/")
dcterms = pm.Namespace("dcterms", "http://purl.org/dc/terms/")
nipype_ns = pm.Namespace("nipype", "http://nipy.org/nipype/terms/")
niiri = pm.Namespace("niiri", "http://iri.nidash.org/")
crypto = pm.Namespace("crypto", ("http://id.loc.gov/vocabulary/preservation/"
                                 "cryptographicHashFunctions/"))
get_id = lambda: niiri[uuid1().hex]

PROV_ENVVARS = [
    'PATH', 'FSLDIR', 'FREESURFER_HOME', 'ANTSPATH', 'CAMINOPATH', 'CLASSPATH',
    'LD_LIBRARY_PATH', 'DYLD_LIBRARY_PATH', 'FIX_VERTEX_AREA',
    'FSF_OUTPUT_FORMAT', 'FSLCONFDIR', 'FSLOUTPUTTYPE', 'LOGNAME', 'USER',
    'MKL_NUM_THREADS', 'OMP_NUM_THREADS'
]


def get_attr_id(attr, skip=None):
    dictwithhash, hashval = get_hashval(attr, skip=skip)
    return niiri[hashval]


max_text_len = 1024000


def get_hashval(inputdict, skip=None):
    """Return a dictionary of our items with hashes for each file.

    Searches through dictionary items and if an item is a file, it
    calculates the md5 hash of the file contents and stores the
    file name and hash value as the new key value.

    However, the overall bunch hash is calculated only on the hash
    value of a file. The path and name of the file are not used in
    the overall hash calculation.

    Returns
    -------
    dict_withhash : dict
        Copy of our dictionary with the new file hashes included
        with each file.
    hashvalue : str
        The md5 hash value of the traited spec

    """

    dict_withhash = {}
    dict_nofilename = OrderedDict()
    keys = {}
    for key in inputdict:
        if skip is not None and key in skip:
            continue
        keys[key.uri] = key
    for key in sorted(keys):
        val = inputdict[keys[key]]
        outname = key
        try:
            if isinstance(val, pm.URIRef):
                val = val.decode()
        except AttributeError:
            pass
        if isinstance(val, pm.QualifiedName):
            val = val.uri
        if isinstance(val, pm.Literal):
            val = val.value
        dict_nofilename[outname] = _get_sorteddict(val)
        dict_withhash[outname] = _get_sorteddict(val, True)
        sorted_dict = str(sorted(dict_nofilename.items()))
    return (dict_withhash, md5(sorted_dict.encode()).hexdigest())


def _get_sorteddict(object, dictwithhash=False):
    if isinstance(object, dict):
        out = OrderedDict()
        for key, val in sorted(object.items()):
            if val:
                out[key] = _get_sorteddict(val, dictwithhash)
    elif isinstance(object, (list, tuple)):
        out = []
        for val in object:
            if val:
                out.append(_get_sorteddict(val, dictwithhash))
        if isinstance(object, tuple):
            out = tuple(out)
    else:
        if isinstance(object, str) and os.path.isfile(object):
            hash = hash_infile(object)
            if dictwithhash:
                out = (object, hash)
            else:
                out = hash
        elif isinstance(object, float):
            out = '%.10f'.format(object)
        else:
            out = object
    return out


def safe_encode(x, as_literal=True):
    """
    Encodes a python value for prov
    """
    if x is None:
        value = "Unknown"
        if as_literal:
            return pm.Literal(value, pm.XSD['string'])
        else:
            return value

    if isinstance(x, (str, bytes)):
        if isinstance(x, bytes):
            x = str(x, 'utf-8')
        if os.path.exists(x):
            if x[0] != os.pathsep:
                x = os.path.abspath(x)
            value = 'file://{}{}'.format(platform.node().lower(), x)
            if not as_literal:
                return value
            try:
                return pm.URIRef(value)
            except AttributeError:
                return pm.Literal(value, pm.XSD['anyURI'])
        else:
            value = x
            if len(x) > max_text_len:
                cliptxt = '...Clipped...'
                value = x[:max_text_len - len(cliptxt)] + cliptxt

            if not as_literal:
                return value

            return pm.Literal(value, pm.XSD['string'])
    if isinstance(x, int):
        if not as_literal:
            return x
        return pm.Literal(int(x), pm.XSD['integer'])
    if isinstance(x, float):
        if not as_literal:
            return x
        return pm.Literal(x, pm.XSD['float'])
    if isinstance(x, dict):
        outdict = {}
        for key, value in list(x.items()):
            encoded_value = safe_encode(value, as_literal=False)
            if isinstance(encoded_value, pm.Literal):
                outdict[key] = encoded_value.json_representation()
            else:
                outdict[key] = encoded_value

        try:
            jsonstr = json.dumps(outdict)
        except UnicodeDecodeError as excp:
            jsonstr = "Could not encode dictionary. {}".format(excp)
            logger.warning('Prov: %s', jsonstr)

        if not as_literal:
            return jsonstr
        return pm.Literal(jsonstr, pm.XSD['string'])
    if isinstance(x, (list, tuple)):
        x = list(x)
        is_object = False
        try:
            nptype = np.array(x).dtype
            is_object = nptype == np.dtype(object)
        except ValueError:
            is_object = True

        # If the array contains an heterogeneous mixture of data types
        # they should be encoded sequentially
        if is_object:
            outlist = []
            for value in x:
                encoded_value = safe_encode(value, as_literal=False)
                if isinstance(encoded_value, pm.Literal):
                    outlist.append(encoded_value.json_representation())
                else:
                    outlist.append(encoded_value)
            x = outlist

        try:
            jsonstr = json.dumps(x)
        except UnicodeDecodeError as excp:
            jsonstr = "Could not encode list/tuple. {}".format(excp)
            logger.warning('Prov: %s', jsonstr)

        if not as_literal:
            return jsonstr
        return pm.Literal(jsonstr, pm.XSD['string'])

    # If is a literal, and as_literal do nothing.
    # else bring back to json.
    if isinstance(x, pm.Literal):
        if as_literal:
            return x
        return dumps(x.json_representation())

    jsonstr = None
    ltype = pm.XSD['string']
    try:
        jsonstr = json.dumps(x.__dict__)
    except AttributeError:
        pass

    if jsonstr is None:
        try:
            jsonstr = dumps(x)
            ltype = nipype_ns['pickle']
        except TypeError as excp:
            jsonstr = 'Could not encode object. {}'.format(excp)

    if not as_literal:
        return jsonstr
    return pm.Literal(jsonstr, ltype)


def prov_encode(graph, value, create_container=True):
    if isinstance(value, (list, tuple)) and create_container:
        value = list(value)
        if len(value) == 0:
            encoded_literal = safe_encode(value)
            attr = {pm.PROV['value']: encoded_literal}
            eid = get_attr_id(attr)
            return graph.entity(eid, attr)

        if len(value) == 1:
            return prov_encode(graph, value[0])

        entities = []
        for item in value:
            item_entity = prov_encode(graph, item)
            entities.append(item_entity)
            if isinstance(item, (list, tuple)):
                continue

            item_entity_val = list(item_entity.value)[0]
            is_str = isinstance(item_entity_val, str)
            if not is_str or (is_str and 'file://' not in item_entity_val):
                return prov_encode(graph, value, create_container=False)

        eid = get_id()
        entity = graph.collection(identifier=eid)
        for item_entity in entities:
            graph.hadMember(eid, item_entity)

        return entity
    else:
        encoded_literal = safe_encode(value)
        attr = {pm.PROV['value']: encoded_literal}
        if isinstance(value, str) and os.path.exists(value):
            attr.update({pm.PROV['location']: encoded_literal})
            if not os.path.isdir(value):
                sha512 = hash_infile(value, crypto=hashlib.sha512)
                attr.update({
                    crypto['sha512']:
                    pm.Literal(sha512, pm.XSD['string'])
                })
                eid = get_attr_id(
                    attr, skip=[pm.PROV['location'], pm.PROV['value']])
            else:
                eid = get_attr_id(attr, skip=[pm.PROV['location']])
        else:
            eid = get_attr_id(attr)
        entity = graph.entity(eid, attr)
    return entity


def write_provenance(results, filename='provenance', format='all'):
    prov = None
    try:
        ps = ProvStore()
        ps.add_results(results)
        prov = ps.write_provenance(filename=filename, format=format)
    except Exception as e:
        import traceback
        err_msg = traceback.format_exc()
        if getattr(e, 'args'):
            err_msg += '\n\nException arguments:\n' + ', '.join(
                ['"%s"' % arg for arg in e.args])
        logger.warning('Writing provenance failed - Exception details:\n%s',
                       err_msg)

    return prov


class ProvStore(object):
    def __init__(self):
        self.g = pm.ProvDocument()
        self.g.add_namespace(foaf)
        self.g.add_namespace(dcterms)
        self.g.add_namespace(nipype_ns)
        self.g.add_namespace(niiri)

    def add_results(self, results, keep_provenance=False):
        if keep_provenance and results.provenance:
            self.g = deepcopy(results.provenance)
            return self.g
        runtime = results.runtime
        interface = results.interface
        inputs = results.inputs
        outputs = results.outputs
        classname = interface.__name__
        modulepath = "{0}.{1}".format(interface.__module__, interface.__name__)
        activitytype = ''.join([i.capitalize() for i in modulepath.split('.')])

        a0_attrs = {
            nipype_ns['module']: interface.__module__,
            nipype_ns["interface"]: classname,
            pm.PROV["type"]: nipype_ns[activitytype],
            pm.PROV["label"]: classname,
            nipype_ns['duration']: safe_encode(runtime.duration),
            nipype_ns['workingDirectory']: safe_encode(runtime.cwd),
            nipype_ns['returnCode']: safe_encode(runtime.returncode),
            nipype_ns['platform']: safe_encode(runtime.platform),
            nipype_ns['version']: safe_encode(runtime.version),
        }
        a0_attrs[foaf["host"]] = pm.Literal(runtime.hostname, pm.XSD['anyURI'])

        try:
            a0_attrs.update({
                nipype_ns['command']: safe_encode(runtime.cmdline)
            })
            a0_attrs.update({
                nipype_ns['commandPath']:
                safe_encode(runtime.command_path)
            })
            a0_attrs.update({
                nipype_ns['dependencies']:
                safe_encode(runtime.dependencies)
            })
        except AttributeError:
            pass
        a0 = self.g.activity(get_id(), runtime.startTime, runtime.endTime,
                             a0_attrs)
        # environment
        id = get_id()
        env_collection = self.g.collection(id)
        env_collection.add_attributes({
            pm.PROV['type']:
            nipype_ns['Environment'],
            pm.PROV['label']:
            "Environment"
        })
        self.g.used(a0, id)
        # write environment entities
        for idx, (key, val) in enumerate(sorted(runtime.environ.items())):
            if key not in PROV_ENVVARS:
                continue
            in_attr = {
                pm.PROV["label"]: key,
                nipype_ns["environmentVariable"]: key,
                pm.PROV["value"]: safe_encode(val)
            }
            id = get_attr_id(in_attr)
            self.g.entity(id, in_attr)
            self.g.hadMember(env_collection, id)
        # write input entities
        if inputs:
            id = get_id()
            input_collection = self.g.collection(id)
            input_collection.add_attributes({
                pm.PROV['type']:
                nipype_ns['Inputs'],
                pm.PROV['label']:
                "Inputs"
            })
            # write input entities
            for idx, (key, val) in enumerate(sorted(inputs.items())):
                in_entity = prov_encode(self.g, val).identifier
                self.g.hadMember(input_collection, in_entity)
                used_attr = {pm.PROV["label"]: key, nipype_ns["inPort"]: key}
                self.g.used(
                    activity=a0, entity=in_entity, other_attributes=used_attr)
        # write output entities
        if outputs:
            id = get_id()
            output_collection = self.g.collection(id)
            if not isinstance(outputs, dict):
                outputs = outputs.get_traitsfree()
            output_collection.add_attributes({
                pm.PROV['type']:
                nipype_ns['Outputs'],
                pm.PROV['label']:
                "Outputs"
            })
            self.g.wasGeneratedBy(output_collection, a0)
            # write output entities
            for idx, (key, val) in enumerate(sorted(outputs.items())):
                out_entity = prov_encode(self.g, val).identifier
                self.g.hadMember(output_collection, out_entity)
                gen_attr = {pm.PROV["label"]: key, nipype_ns["outPort"]: key}
                self.g.generation(
                    out_entity, activity=a0, other_attributes=gen_attr)
        # write runtime entities
        id = get_id()
        runtime_collection = self.g.collection(id)
        runtime_collection.add_attributes({
            pm.PROV['type']:
            nipype_ns['Runtime'],
            pm.PROV['label']:
            "RuntimeInfo"
        })
        self.g.wasGeneratedBy(runtime_collection, a0)
        for key, value in sorted(runtime.items()):
            if not value:
                continue
            if key not in ['stdout', 'stderr', 'merged']:
                continue
            attr = {pm.PROV["label"]: key, nipype_ns[key]: safe_encode(value)}
            id = get_id()
            self.g.entity(get_id(), attr)
            self.g.hadMember(runtime_collection, id)

        # create agents
        user_attr = {
            pm.PROV["type"]: pm.PROV["Person"],
            pm.PROV["label"]: getpass.getuser(),
            foaf["name"]: safe_encode(getpass.getuser())
        }
        user_agent = self.g.agent(get_attr_id(user_attr), user_attr)
        agent_attr = {
            pm.PROV["type"]: pm.PROV["SoftwareAgent"],
            pm.PROV["label"]: "Nipype",
            foaf["name"]: safe_encode("Nipype"),
            nipype_ns["version"]: __version__
        }
        for key, value in list(get_info().items()):
            agent_attr.update({nipype_ns[key]: safe_encode(value)})
        software_agent = self.g.agent(get_attr_id(agent_attr), agent_attr)
        self.g.wasAssociatedWith(a0, user_agent, None, None, {
            pm.PROV["hadRole"]: nipype_ns["LoggedInUser"]
        })
        self.g.wasAssociatedWith(a0, software_agent)
        return self.g

    def write_provenance(self, filename='provenance', format='all'):
        if format in ['provn', 'all']:
            with open(filename + '.provn', 'wt') as fp:
                fp.writelines(self.g.get_provn())
        try:
            if format in ['rdf', 'all']:
                if len(self.g.bundles) == 0:
                    rdf_format = 'turtle'
                    ext = '.ttl'
                else:
                    rdf_format = 'trig'
                    ext = '.trig'
                self.g.serialize(
                    filename + ext, format='rdf', rdf_format=rdf_format)
            if format in ['jsonld']:
                self.g.serialize(
                    filename + '.jsonld',
                    format='rdf',
                    rdf_format='json-ld',
                    indent=4)
        except pm.serializers.DoNotExist:
            pass
        return self.g



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import Realign


def test_Realign_inputs():
    input_map = dict(
        fwhm=dict(field='eoptions.fwhm', ),
        in_files=dict(
            copyfile=True,
            field='data',
            mandatory=True,
        ),
        interp=dict(field='eoptions.interp', ),
        jobtype=dict(usedefault=True, ),
        matlab_cmd=dict(),
        mfile=dict(usedefault=True, ),
        out_prefix=dict(
            field='roptions.prefix',
            usedefault=True,
        ),
        paths=dict(),
        quality=dict(field='eoptions.quality', ),
        register_to_mean=dict(field='eoptions.rtm', ),
        separation=dict(field='eoptions.sep', ),
        use_mcr=dict(),
        use_v8struct=dict(
            min_ver='8',
            usedefault=True,
        ),
        weight_img=dict(
            extensions=None,
            field='eoptions.weight',
        ),
        wrap=dict(field='eoptions.wrap', ),
        write_interp=dict(field='roptions.interp', ),
        write_mask=dict(field='roptions.mask', ),
        write_which=dict(
            field='roptions.which',
            maxlen=2,
            minlen=2,
            usedefault=True,
        ),
        write_wrap=dict(field='roptions.wrap', ),
    )
    inputs = Realign.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Realign_outputs():
    output_map = dict(
        mean_image=dict(extensions=None, ),
        modified_in_files=dict(),
        realigned_files=dict(),
        realignment_parameters=dict(),
    )
    outputs = Realign.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



from .. import get_info
from ..info import get_nipype_gitversion
import pytest


def test_nipype_info():
    exception_not_raised = True
    try:
        get_info()
    except Exception:
        exception_not_raised = False
    assert exception_not_raised


@pytest.mark.skipif(not get_nipype_gitversion(),
                    reason="not able to get version from get_nipype_gitversion")
def test_git_hash():
    # removing the first "g" from gitversion
    get_nipype_gitversion()[1:] == get_info()['commit_hash']



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import SegmentCC


def test_SegmentCC_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        copy_inputs=dict(),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='-aseg %s',
            extensions=None,
            mandatory=True,
        ),
        in_norm=dict(
            extensions=None,
            mandatory=True,
        ),
        out_file=dict(
            argstr='-o %s',
            extensions=None,
            hash_files=False,
            keep_extension=False,
            name_source=['in_file'],
            name_template='%s.auto.mgz',
        ),
        out_rotation=dict(
            argstr='-lta %s',
            extensions=None,
            mandatory=True,
        ),
        subject_id=dict(
            argstr='%s',
            mandatory=True,
            position=-1,
            usedefault=True,
        ),
        subjects_dir=dict(),
    )
    inputs = SegmentCC.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_SegmentCC_outputs():
    output_map = dict(
        out_file=dict(extensions=None, ),
        out_rotation=dict(extensions=None, ),
    )
    outputs = SegmentCC.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..utils import Tkregister2


def test_Tkregister2_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fsl_in_matrix=dict(
            argstr='--fsl %s',
            extensions=None,
        ),
        fsl_out=dict(argstr='--fslregout %s', ),
        fstal=dict(
            argstr='--fstal',
            xor=['target_image', 'moving_image', 'reg_file'],
        ),
        fstarg=dict(
            argstr='--fstarg',
            xor=['target_image'],
        ),
        invert_lta_in=dict(requires=['lta_in'], ),
        invert_lta_out=dict(
            argstr='--ltaout-inv',
            requires=['lta_in'],
        ),
        lta_in=dict(
            argstr='--lta %s',
            extensions=None,
        ),
        lta_out=dict(argstr='--ltaout %s', ),
        moving_image=dict(
            argstr='--mov %s',
            extensions=None,
            mandatory=True,
        ),
        movscale=dict(argstr='--movscale %f', ),
        noedit=dict(
            argstr='--noedit',
            usedefault=True,
        ),
        reg_file=dict(
            argstr='--reg %s',
            extensions=None,
            mandatory=True,
            usedefault=True,
        ),
        reg_header=dict(argstr='--regheader', ),
        subject_id=dict(argstr='--s %s', ),
        subjects_dir=dict(),
        target_image=dict(
            argstr='--targ %s',
            extensions=None,
            xor=['fstarg'],
        ),
        xfm=dict(
            argstr='--xfm %s',
            extensions=None,
        ),
    )
    inputs = Tkregister2.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Tkregister2_outputs():
    output_map = dict(
        fsl_file=dict(extensions=None, ),
        lta_file=dict(extensions=None, ),
        reg_file=dict(extensions=None, ),
    )
    outputs = Tkregister2.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..em import EM


def test_EM_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        bc_order_val=dict(
            argstr='-bc_order %s',
            usedefault=True,
        ),
        bc_thresh_val=dict(
            argstr='-bc_thresh %s',
            usedefault=True,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='-in %s',
            extensions=None,
            mandatory=True,
            position=4,
        ),
        mask_file=dict(
            argstr='-mask %s',
            extensions=None,
        ),
        max_iter=dict(
            argstr='-max_iter %s',
            usedefault=True,
        ),
        min_iter=dict(
            argstr='-min_iter %s',
            usedefault=True,
        ),
        mrf_beta_val=dict(argstr='-mrf_beta %s', ),
        no_prior=dict(
            argstr='-nopriors %s',
            mandatory=True,
            xor=['prior_4D', 'priors'],
        ),
        out_bc_file=dict(
            argstr='-bc_out %s',
            extensions=None,
            name_source=['in_file'],
            name_template='%s_bc_em.nii.gz',
        ),
        out_file=dict(
            argstr='-out %s',
            extensions=None,
            name_source=['in_file'],
            name_template='%s_em.nii.gz',
        ),
        out_outlier_file=dict(
            argstr='-out_outlier %s',
            extensions=None,
            name_source=['in_file'],
            name_template='%s_outlier_em.nii.gz',
        ),
        outlier_val=dict(argstr='-outlier %s %s', ),
        prior_4D=dict(
            argstr='-prior4D %s',
            extensions=None,
            mandatory=True,
            xor=['no_prior', 'priors'],
        ),
        priors=dict(
            argstr='%s',
            mandatory=True,
            xor=['no_prior', 'prior_4D'],
        ),
        reg_val=dict(argstr='-reg %s', ),
        relax_priors=dict(argstr='-rf %s %s', ),
    )
    inputs = EM.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_EM_outputs():
    output_map = dict(
        out_bc_file=dict(extensions=None, ),
        out_file=dict(extensions=None, ),
        out_outlier_file=dict(extensions=None, ),
    )
    outputs = EM.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# emacs: -*- coding: utf-8; mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set fileencoding=utf-8 ft=python sts=4 ts=4 sw=4 et:
#
# nipype documentation build configuration file, created by
# sphinx-quickstart on Mon Jul 20 12:30:18 2009.
#
# This file is execfile()d with the current directory set to its containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

import sys
import os
from shutil import rmtree

nipypepath = os.path.abspath('..')
sys.path.insert(1, nipypepath)

import nipype

if not os.path.exists('users/examples'):
    os.mkdir('users/examples')
os.system('python ../tools/make_examples.py --no-exec')

if os.path.exists('api/generated'):
    rmtree('api/generated')
os.system('python ../tools/build_modref_templates.py')
if os.path.exists('interfaces/generated'):
    rmtree('interfaces/generated')
os.system('python ../tools/build_interface_docs.py')

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
sys.path.append(os.path.abspath('sphinxext'))

# -- General configuration -----------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be extensions
# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = ['sphinx.ext.todo',
              'sphinx.ext.imgmath',
              'sphinx.ext.inheritance_diagram',
              'sphinx.ext.graphviz',
              'sphinx.ext.autodoc',
              'sphinx.ext.doctest',
              'sphinx.ext.autosummary',
              'numpydoc',
              'matplotlib.sphinxext.plot_directive',
              #'matplotlib.sphinxext.only_directives',
              'nipype.sphinxext.plot_workflow',
              #'IPython.sphinxext.ipython_directive',
              #'IPython.sphinxext.ipython_console_highlighting'
              ]
on_rtd = os.environ.get('READTHEDOCS') == 'True'
if on_rtd:
    extensions.append('readthedocs_ext.readthedocs')

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The encoding of source files.
#source_encoding = 'utf-8'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = u'nipype'
copyright = u'2009-19, Neuroimaging in Python team'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = nipype.__version__
# The full version, including alpha/beta/rc tags.
release = "1.3.0-rc1"

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#language = None

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
#today = ''
# Else, today_fmt is used as the format for a strftime call.
today_fmt = '%B %d, %Y, %H:%M PDT'

# List of documents that shouldn't be included in the build.
unused_docs = ['api/generated/gen']

# List of directories, relative to source directory, that shouldn't be searched
# for source files.
exclude_trees = ['_build']

# The reST default role (used for this markup: `text`) to use for all documents.
#default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
#add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
#add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
#show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# A list of ignored prefixes for module index sorting.
#modindex_common_prefix = []

# -- Sphinxext configuration ---------------------------------------------------

# Set attributes for layout of inheritance diagrams
inheritance_graph_attrs = dict(rankdir="LR", size='"6.0, 8.0"', fontsize=14,
                               ratio='compress')
inheritance_node_attrs = dict(shape='ellipse', fontsize=14, height=0.75,
                              color='dodgerblue1', style='filled')

# Flag to show todo items in rendered output
todo_include_todos = True

# -- Options for HTML output ---------------------------------------------------

# The theme to use for HTML and HTML Help pages.  Major themes that come with
# Sphinx are currently 'default' and 'sphinxdoc'.
html_theme = 'sphinxdoc'

# The style sheet to use for HTML and HTML Help pages. A file of that name
# must exist either in Sphinx' static/ path, or in one of the custom paths
# given in html_static_path.
html_style = 'nipype.css'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#html_theme_options = {}

# Add any paths that contain custom themes here, relative to this directory.
#html_theme_path = []

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
html_title = 'nipy pipeline and interfaces package'

# A shorter title for the navigation bar.  Default is the same as html_title.
#html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
#html_logo = None

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
#html_favicon = None

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
#html_last_updated_fmt = '%b %d, %Y'

# Content template for the index page.
html_index = 'index.html'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
#html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
html_sidebars = {'**': ['gse.html', 'localtoc.html', 'sidebar_versions.html', 'indexsidebar.html'],
                 'searchresults': ['sidebar_versions.html', 'indexsidebar.html'],
                 'version': []}

# Additional templates that should be rendered to pages, maps page names to
# template names.
#html_additional_pages = {'index': 'index.html'}

# If false, no module index is generated.
#html_use_modindex = True

# If false, no index is generated.
#html_use_index = True

# If true, the index is split into individual pages for each letter.
#html_split_index = False

# If true, links to the reST sources are added to the pages.
html_show_sourcelink = False

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
#html_use_opensearch = ''

# If nonempty, this is the file name suffix for HTML files (e.g. ".xhtml").
#html_file_suffix = ''

# Output file base name for HTML help builder.
htmlhelp_basename = 'nipypedoc'


# -- Options for LaTeX output --------------------------------------------------

# The paper size ('letter' or 'a4').
#latex_paper_size = 'letter'

# The font size ('10pt', '11pt' or '12pt').
#latex_font_size = '10pt'

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass [howto/manual]).
latex_documents = [
    ('documentation', 'nipype.tex', u'nipype Documentation',
     u'Neuroimaging in Python team', 'manual'),
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
#latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
#latex_use_parts = False

# Additional stuff for the LaTeX preamble.
#latex_preamble = ''

# Documents to append as an appendix to all manuals.
#latex_appendices = []

# If false, no module index is generated.
#latex_use_modindex = True


# Example configuration for intersphinx: refer to the Python standard library.
intersphinx_mapping = {'http://docs.python.org/': None}

exclude_patterns = ['interfaces/generated/gen.rst', 'api/generated/gen.rst']



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import Hist


def test_Hist_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        bin_width=dict(argstr='-binwidth %f', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr='-input %s',
            copyfile=False,
            extensions=None,
            mandatory=True,
            position=1,
        ),
        mask=dict(
            argstr='-mask %s',
            extensions=None,
        ),
        max_value=dict(argstr='-max %f', ),
        min_value=dict(argstr='-min %f', ),
        nbin=dict(argstr='-nbin %d', ),
        out_file=dict(
            argstr='-prefix %s',
            extensions=None,
            keep_extension=False,
            name_source=['in_file'],
            name_template='%s_hist',
        ),
        out_show=dict(
            argstr='> %s',
            extensions=None,
            keep_extension=False,
            name_source='in_file',
            name_template='%s_hist.out',
            position=-1,
        ),
        showhist=dict(
            argstr='-showhist',
            usedefault=True,
        ),
    )
    inputs = Hist.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_Hist_outputs():
    output_map = dict(
        out_file=dict(extensions=None, ),
        out_show=dict(extensions=None, ),
    )
    outputs = Hist.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value




from distutils.version import LooseVersion
from ... import logging
from .base import (HAVE_DIPY, dipy_version, dipy_to_nipype_interface,
                   get_dipy_workflows)

IFLOGGER = logging.getLogger('nipype.interface')


if HAVE_DIPY and LooseVersion(dipy_version()) >= LooseVersion('0.15'):
    from dipy.workflows import align

    l_wkflw = get_dipy_workflows(align)
    for name, obj in l_wkflw:
        new_name = name.replace('Flow', '')
        globals()[new_name] = dipy_to_nipype_interface(new_name, obj)
    del l_wkflw

else:
    IFLOGGER.info("We advise you to upgrade DIPY version. This upgrade will"
                  " open access to more function")



#!/usr/bin/env python
"""Run the py->rst conversion and run all examples.

This also creates the index.rst file appropriately, makes figures, etc.
"""
from __future__ import (print_function, division, unicode_literals,
                        absolute_import)
from builtins import open
from past.builtins import execfile
# -----------------------------------------------------------------------------
# Library imports
# -----------------------------------------------------------------------------

# Stdlib imports
import os
import sys

from glob import glob

# Third-party imports

# We must configure the mpl backend before making any further mpl imports
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt

from matplotlib._pylab_helpers import Gcf

# Local tools
from toollib import *

# -----------------------------------------------------------------------------
# Globals
# -----------------------------------------------------------------------------

examples_header = """

.. _examples:

Examples
========

.. note_about_examples
"""
# -----------------------------------------------------------------------------
# Function defintions
# -----------------------------------------------------------------------------

# These global variables let show() be called by the scripts in the usual
# manner, but when generating examples, we override it to write the figures to
# files with a known name (derived from the script name) plus a counter
figure_basename = None

# We must change the show command to save instead


def show():
    allfm = Gcf.get_all_fig_managers()
    for fcount, fm in enumerate(allfm):
        fm.canvas.figure.savefig('%s_%02i.png' % (figure_basename, fcount + 1))


_mpl_show = plt.show
plt.show = show

# -----------------------------------------------------------------------------
# Main script
# -----------------------------------------------------------------------------

# Work in examples directory
cd('users/examples')
if not os.getcwd().endswith('users/examples'):
    raise OSError('This must be run from doc/examples directory')

# Run the conversion from .py to rst file
sh('../../../tools/ex2rst --project Nipype --outdir . ../../../examples')
sh('../../../tools/ex2rst --project Nipype '
   '--outdir . ../../../examples/frontiers_paper')

# Make the index.rst file
"""
index = open('index.rst', 'w')
index.write(examples_header)
for name in [os.path.splitext(f)[0] for f in glob('*.rst')]:
    #Don't add the index in there to avoid sphinx errors and don't add the
    #note_about examples again (because it was added at the top):
    if name not in(['index','note_about_examples']):
        index.write('   %s\n' % name)
index.close()
"""

# Execute each python script in the directory.
if '--no-exec' in sys.argv:
    pass
else:
    if not os.path.isdir('fig'):
        os.mkdir('fig')

    for script in glob('*.py'):
        figure_basename = pjoin('fig', os.path.splitext(script)[0])
        execfile(script)
        plt.close('all')



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

import os

from ...base import (CommandLine, CommandLineInputSpec, SEMLikeCommandLine,
                     TraitedSpec, File, Directory, traits, isdefined,
                     InputMultiPath, OutputMultiPath)


class BRAINSResampleInputSpec(CommandLineInputSpec):
    inputVolume = File(
        desc="Image To Warp", exists=True, argstr="--inputVolume %s")
    referenceVolume = File(
        desc=
        "Reference image used only to define the output space. If not specified, the warping is done in the same space as the image to warp.",
        exists=True,
        argstr="--referenceVolume %s")
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Resulting deformed image",
        argstr="--outputVolume %s")
    pixelType = traits.Enum(
        "float",
        "short",
        "ushort",
        "int",
        "uint",
        "uchar",
        "binary",
        desc=
        "Specifies the pixel type for the input/output images.  The \'binary\' pixel type uses a modified algorithm whereby the image is read in as unsigned char, a signed distance map is created, signed distance map is resampled, and then a thresholded image of type unsigned char is written to disk.",
        argstr="--pixelType %s")
    deformationVolume = File(
        desc=
        "Displacement Field to be used to warp the image (ITKv3 or earlier)",
        exists=True,
        argstr="--deformationVolume %s")
    warpTransform = File(
        desc=
        "Filename for the BRAINSFit transform (ITKv3 or earlier) or composite transform file (ITKv4)",
        exists=True,
        argstr="--warpTransform %s")
    interpolationMode = traits.Enum(
        "NearestNeighbor",
        "Linear",
        "ResampleInPlace",
        "BSpline",
        "WindowedSinc",
        "Hamming",
        "Cosine",
        "Welch",
        "Lanczos",
        "Blackman",
        desc=
        "Type of interpolation to be used when applying transform to moving volume.  Options are Linear, ResampleInPlace, NearestNeighbor, BSpline, or WindowedSinc",
        argstr="--interpolationMode %s")
    inverseTransform = traits.Bool(
        desc=
        "True/False is to compute inverse of given transformation. Default is false",
        argstr="--inverseTransform ")
    defaultValue = traits.Float(
        desc="Default voxel value", argstr="--defaultValue %f")
    gridSpacing = InputMultiPath(
        traits.Int,
        desc=
        "Add warped grid to output image to help show the deformation that occured with specified spacing.   A spacing of 0 in a dimension indicates that grid lines should be rendered to fall exactly (i.e. do not allow displacements off that plane).  This is useful for makeing a 2D image of grid lines from the 3D space",
        sep=",",
        argstr="--gridSpacing %s")
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d")


class BRAINSResampleOutputSpec(TraitedSpec):
    outputVolume = File(desc="Resulting deformed image", exists=True)


class BRAINSResample(SEMLikeCommandLine):
    """title: Resample Image (BRAINS)

category: Registration

description: This program collects together three common image processing tasks that all involve resampling an image volume: Resampling to a new resolution and spacing, applying a transformation (using an ITK transform IO mechanisms) and Warping (using a vector image deformation field).  Full documentation available here: http://wiki.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/BRAINSResample.

version: 3.0.0

documentation-url: http://www.slicer.org/slicerWiki/index.php/Documentation/4.1/Modules/BRAINSResample

license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

contributor: This tool was developed by Vincent Magnotta, Greg Harris, and Hans Johnson.

acknowledgements: The development of this tool was supported by funding from grants NS050568 and NS40068 from the National Institute of Neurological Disorders and Stroke and grants MH31593, MH40856, from the National Institute of Mental Health.

"""

    input_spec = BRAINSResampleInputSpec
    output_spec = BRAINSResampleOutputSpec
    _cmd = " BRAINSResample "
    _outputs_filenames = {'outputVolume': 'outputVolume.nii'}
    _redirect_x = False



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..preprocess import FUGUE


def test_FUGUE_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        asym_se_time=dict(argstr='--asym=%.10f', ),
        despike_2dfilter=dict(argstr='--despike', ),
        despike_threshold=dict(argstr='--despikethreshold=%s', ),
        dwell_time=dict(argstr='--dwell=%.10f', ),
        dwell_to_asym_ratio=dict(argstr='--dwelltoasym=%.10f', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fmap_in_file=dict(
            argstr='--loadfmap=%s',
            extensions=None,
        ),
        fmap_out_file=dict(
            argstr='--savefmap=%s',
            extensions=None,
        ),
        forward_warping=dict(usedefault=True, ),
        fourier_order=dict(argstr='--fourier=%d', ),
        icorr=dict(
            argstr='--icorr',
            requires=['shift_in_file'],
        ),
        icorr_only=dict(
            argstr='--icorronly',
            requires=['unwarped_file'],
        ),
        in_file=dict(
            argstr='--in=%s',
            extensions=None,
        ),
        mask_file=dict(
            argstr='--mask=%s',
            extensions=None,
        ),
        median_2dfilter=dict(argstr='--median', ),
        no_extend=dict(argstr='--noextend', ),
        no_gap_fill=dict(argstr='--nofill', ),
        nokspace=dict(argstr='--nokspace', ),
        output_type=dict(),
        pava=dict(argstr='--pava', ),
        phase_conjugate=dict(argstr='--phaseconj', ),
        phasemap_in_file=dict(
            argstr='--phasemap=%s',
            extensions=None,
        ),
        poly_order=dict(argstr='--poly=%d', ),
        save_fmap=dict(xor=['save_unmasked_fmap'], ),
        save_shift=dict(xor=['save_unmasked_shift'], ),
        save_unmasked_fmap=dict(
            argstr='--unmaskfmap',
            xor=['save_fmap'],
        ),
        save_unmasked_shift=dict(
            argstr='--unmaskshift',
            xor=['save_shift'],
        ),
        shift_in_file=dict(
            argstr='--loadshift=%s',
            extensions=None,
        ),
        shift_out_file=dict(
            argstr='--saveshift=%s',
            extensions=None,
        ),
        smooth2d=dict(argstr='--smooth2=%.2f', ),
        smooth3d=dict(argstr='--smooth3=%.2f', ),
        unwarp_direction=dict(argstr='--unwarpdir=%s', ),
        unwarped_file=dict(
            argstr='--unwarp=%s',
            extensions=None,
            requires=['in_file'],
            xor=['warped_file'],
        ),
        warped_file=dict(
            argstr='--warp=%s',
            extensions=None,
            requires=['in_file'],
            xor=['unwarped_file'],
        ),
    )
    inputs = FUGUE.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_FUGUE_outputs():
    output_map = dict(
        fmap_out_file=dict(extensions=None, ),
        shift_out_file=dict(extensions=None, ),
        unwarped_file=dict(extensions=None, ),
        warped_file=dict(extensions=None, ),
    )
    outputs = FUGUE.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value
